/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
import { Queryable, HierarchyIterable } from "../types";
/**
 * Creates a [[HierarchyIterable]] for the elements of `source` with the provided `value` appended to the
 * end.
 *
 * @param source The [[HierarchyIterable]] to append to.
 * @param value The value to append.
 * @category Subquery
 */
export declare function append<TNode, T extends TNode>(source: HierarchyIterable<TNode, T>, value: T): HierarchyIterable<TNode, T>;
/**
 * Creates an [[Iterable]] for the elements of `source` with the provided `value` appended to the
 * end.
 *
 * @param source The [[Queryable]] to append to.
 * @param value The value to append.
 * @category Subquery
 */
export declare function append<T>(source: Queryable<T>, value: T): Iterable<T>;
