"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
/**
 * Creates an [[AsyncIterable]] that, when iterated, consumes the provided [[AsyncIterator]].
 *
 * @param iterator An [[AsyncIterator]] object.
 * @category Query
 */
function consumeAsync(iterator, { cacheElements = false, leaveOpen = false } = {}) {
    internal_1.assert.mustBeIterator(iterator, "iterator");
    internal_1.assert.mustBeBoolean(cacheElements, "cacheElements");
    return new AsyncConsumeIterable(iterator, cacheElements, leaveOpen);
}
exports.consumeAsync = consumeAsync;
let AsyncConsumeIterable = class AsyncConsumeIterable {
    constructor(iterator, cacheElements, leaveOpen) {
        this._iterator = iterator;
        this._cache = cacheElements ? [] : undefined;
        this._leaveOpen = leaveOpen;
    }
    [Symbol.asyncIterator]() {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            let offset = 0;
            try {
                for (;;) {
                    if (this._cache && offset < this._cache.length) {
                        yield yield tslib_1.__await(this._cache[offset++]);
                        continue;
                    }
                    else if (this._iterator) {
                        const { done, value } = yield tslib_1.__await(this._iterator.next());
                        if (!done) {
                            if (this._cache)
                                this._cache.push(value);
                            yield yield tslib_1.__await(value);
                            offset++;
                            continue;
                        }
                        this._iterator = undefined;
                    }
                    return yield tslib_1.__await(void 0);
                }
            }
            finally {
                if (!this._leaveOpen) {
                    yield tslib_1.__await(internal_1.AsyncIteratorClose(this._iterator));
                }
            }
        });
    }
};
AsyncConsumeIterable = tslib_1.__decorate([
    internal_1.ToStringTag("AsyncConsumeIterable")
], AsyncConsumeIterable);
