"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
function defaultIfEmpty(source, defaultValue) {
    internal_1.assert.mustBeQueryable(source, "source");
    return internal_1.FlowHierarchy(new DefaultIfEmptyIterable(internal_1.ToIterable(source), defaultValue), source);
}
exports.defaultIfEmpty = defaultIfEmpty;
let DefaultIfEmptyIterable = class DefaultIfEmptyIterable {
    constructor(source, defaultValue) {
        this._source = source;
        this._defaultValue = defaultValue;
    }
    *[Symbol.iterator]() {
        const source = this._source;
        const defaultValue = this._defaultValue;
        let hasElements = false;
        for (const value of source) {
            hasElements = true;
            yield value;
        }
        if (!hasElements) {
            yield defaultValue;
        }
    }
};
DefaultIfEmptyIterable = tslib_1.__decorate([
    internal_1.ToStringTag("DefaultIfEmptyIterable")
], DefaultIfEmptyIterable);
