/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
import { PossiblyAsyncHierarchyIterable, AsyncHierarchyIterable, AsyncQueryable } from "../types";
/**
 * Creates an [[AsyncHierarchyIterable]] that contains the provided default value if `source`
 * contains no elements.
 *
 * @param source A [[HierarchyIterable]] or [[AsyncHierarchyIterable]] object.
 * @param defaultValue The default value.
 * @category Subquery
 */
export declare function defaultIfEmptyAsync<TNode, T extends TNode>(source: PossiblyAsyncHierarchyIterable<TNode, T>, defaultValue: PromiseLike<T> | T): AsyncHierarchyIterable<TNode, T>;
/**
 * Creates an [[AsyncIterable]] that contains the provided default value if `source`
 * contains no elements.
 *
 * @param source An [[AsyncQueryable]] object.
 * @param defaultValue The default value.
 * @category Subquery
 */
export declare function defaultIfEmptyAsync<T>(source: AsyncQueryable<T>, defaultValue: PromiseLike<T> | T): AsyncIterable<T>;
