"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
function flatMap(source, projection, resultSelector) {
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeFunction(projection, "projection");
    internal_1.assert.mustBeFunctionOrUndefined(resultSelector, "resultSelector");
    return new FlatMapIterable(internal_1.ToIterable(source), projection, resultSelector);
}
exports.flatMap = flatMap;
let FlatMapIterable = class FlatMapIterable {
    constructor(source, projection, resultSelector) {
        this._source = source;
        this._projection = projection;
        this._resultSelector = resultSelector;
    }
    *[Symbol.iterator]() {
        const projection = this._projection;
        const resultSelector = this._resultSelector;
        for (const element of this._source) {
            const inner = projection(element);
            if (resultSelector) {
                for (const innerElement of internal_1.ToIterable(inner)) {
                    yield resultSelector(element, innerElement);
                }
            }
            else {
                yield* internal_1.ToIterable(inner);
            }
        }
    }
};
FlatMapIterable = tslib_1.__decorate([
    internal_1.ToStringTag("FlatMapIterable")
], FlatMapIterable);
