"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
/**
 * Creates an [[AsyncIterable]] whose values are provided by a callback executed a provided number of
 * times.
 *
 * @param count The number of times to execute the callback.
 * @param generator The callback to execute.
 * @category Query
 */
function generateAsync(count, generator) {
    internal_1.assert.mustBePositiveFiniteNumber(count, "count");
    internal_1.assert.mustBeFunction(generator, "generator");
    return new AsyncGenerateIterable(count, generator);
}
exports.generateAsync = generateAsync;
let AsyncGenerateIterable = class AsyncGenerateIterable {
    constructor(count, generator) {
        this._count = count;
        this._generator = generator;
    }
    [Symbol.asyncIterator]() {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            const count = this._count;
            const generator = this._generator;
            for (let i = 0; i < count; i++) {
                yield yield tslib_1.__await(generator(i));
            }
        });
    }
};
AsyncGenerateIterable = tslib_1.__decorate([
    internal_1.ToStringTag("AsyncGenerateIterable")
], AsyncGenerateIterable);
