"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const common_1 = require("./common");
function groupByAsync(source, keySelector, elementSelector = common_1.identity, resultSelector = internal_1.CreateGrouping) {
    internal_1.assert.mustBeAsyncQueryable(source, "source");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    internal_1.assert.mustBeFunction(elementSelector, "elementSelector");
    internal_1.assert.mustBeFunction(resultSelector, "resultSelector");
    return new AsyncGroupByIterable(internal_1.ToPossiblyAsyncIterable(source), keySelector, elementSelector, resultSelector);
}
exports.groupByAsync = groupByAsync;
let AsyncGroupByIterable = class AsyncGroupByIterable {
    constructor(source, keySelector, elementSelector, resultSelector) {
        this._source = source;
        this._keySelector = keySelector;
        this._elementSelector = elementSelector;
        this._resultSelector = resultSelector;
    }
    [Symbol.asyncIterator]() {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            const source = this._source;
            const elementSelector = this._elementSelector;
            const resultSelector = this._resultSelector;
            const map = yield tslib_1.__await(internal_1.CreateGroupingsAsync(source, this._keySelector, this._elementSelector));
            for (const [key, values] of map) {
                yield yield tslib_1.__await(resultSelector(key, elementSelector === common_1.identity ? internal_1.FlowHierarchy(values, source) : values));
            }
        });
    }
};
AsyncGroupByIterable = tslib_1.__decorate([
    internal_1.ToStringTag("AsyncGroupByIterable")
], AsyncGroupByIterable);
