"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const empty_1 = require("./empty");
/**
 * Creates an [[AsyncIterable]] that iterates the elements from one of two sources based on the result of a
 * lazily evaluated condition.
 *
 * @param condition A callback used to choose a source.
 * @param thenQueryable The source to use when the callback evaluates to `true`.
 * @param elseQueryable The source to use when the callback evaluates to `false`.
 * @category Query
 */
function ifAsync(condition, thenQueryable, elseQueryable) {
    internal_1.assert.mustBeFunction(condition, "condition");
    internal_1.assert.mustBeAsyncQueryable(thenQueryable, "thenQueryable");
    internal_1.assert.mustBeAsyncQueryableOrUndefined(elseQueryable, "elseQueryable");
    return new AsyncIfIterable(condition, internal_1.ToPossiblyAsyncIterable(thenQueryable), elseQueryable && internal_1.ToPossiblyAsyncIterable(elseQueryable));
}
exports.ifAsync = ifAsync;
let AsyncIfIterable = class AsyncIfIterable {
    constructor(condition, thenQueryable, elseQueryable) {
        this._condition = condition;
        this._thenQueryable = thenQueryable;
        this._elseQueryable = elseQueryable;
    }
    [Symbol.asyncIterator]() {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            const condition = this._condition;
            const result = condition();
            const iterable = (typeof result === "boolean" ? result : yield tslib_1.__await(result))
                ? this._thenQueryable
                : this._elseQueryable || empty_1.empty();
            yield tslib_1.__await(yield* tslib_1.__asyncDelegator(tslib_1.__asyncValues(iterable)));
        });
    }
};
AsyncIfIterable = tslib_1.__decorate([
    internal_1.ToStringTag("AsyncIfIterable")
], AsyncIfIterable);
