"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const toArrayAsync_1 = require("./toArrayAsync");
function includesSequenceAsync(left, right, equalityComparison = internal_1.SameValue) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        var e_1, _a;
        internal_1.assert.mustBeAsyncQueryable(left, "source");
        internal_1.assert.mustBeAsyncQueryable(right, "other");
        internal_1.assert.mustBeFunction(equalityComparison, "equalityComparison");
        const rightArray = yield toArrayAsync_1.toArrayAsync(right);
        const numRightElements = rightArray.length;
        if (numRightElements <= 0) {
            return true;
        }
        const span = [];
        try {
            for (var _b = tslib_1.__asyncValues(internal_1.ToPossiblyAsyncIterable(left)), _c; _c = yield _b.next(), !_c.done;) {
                const leftValue = _c.value;
                for (;;) {
                    const rightValue = rightArray[span.length];
                    if (equalityComparison(leftValue, rightValue)) {
                        if (span.length + 1 >= numRightElements) {
                            return true;
                        }
                        span.push(leftValue);
                    }
                    else if (span.length > 0) {
                        span.shift();
                        continue;
                    }
                    break;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return false;
    });
}
exports.includesSequenceAsync = includesSequenceAsync;
