"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
/**
 * Creates an [[Iterable]] for the own property values of an `object`.
 *
 * @param source An `object`.
 * @category Query
 */
function objectValues(source) {
    internal_1.assert.mustBeObject(source, "source");
    return new ObjectValuesIterable(source);
}
exports.objectValues = objectValues;
let ObjectValuesIterable = class ObjectValuesIterable {
    constructor(source) {
        this._source = source;
    }
    *[Symbol.iterator]() {
        const source = this._source;
        for (const key of Object.keys(source)) {
            yield source[key];
        }
    }
};
ObjectValuesIterable = tslib_1.__decorate([
    internal_1.ToStringTag("ObjectValuesIterable")
], ObjectValuesIterable);
