"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const toArrayAsync_1 = require("./toArrayAsync");
const common_1 = require("./common");
function reduceRightAsync(source, accumulator, seed, resultSelector = common_1.identity) {
    return reduceRightAsyncCore(source, accumulator, seed, arguments.length > 2, resultSelector);
}
exports.reduceRightAsync = reduceRightAsync;
function reduceRightAsyncCore(source, accumulator, current, hasCurrent, resultSelector) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        internal_1.assert.mustBeAsyncQueryable(source, "source");
        internal_1.assert.mustBeFunction(accumulator, "accumulator");
        internal_1.assert.mustBeFunction(resultSelector, "resultSelector");
        const sourceArray = yield toArrayAsync_1.toArrayAsync(source);
        let count = 0;
        for (let offset = sourceArray.length - 1; offset >= 0; offset--) {
            const value = sourceArray[offset];
            if (!hasCurrent) {
                current = value;
                hasCurrent = true;
            }
            else {
                current = yield accumulator(current, value, offset);
            }
            count++;
        }
        return resultSelector(current, count);
    });
}
