"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const toArrayAsync_1 = require("./toArrayAsync");
function scanRightAsync(source, accumulator, seed) {
    internal_1.assert.mustBeAsyncQueryable(source, "source");
    internal_1.assert.mustBeFunction(accumulator, "accumulator");
    return new AsyncScanRightIterable(internal_1.ToPossiblyAsyncIterable(source), accumulator, arguments.length > 2, seed);
}
exports.scanRightAsync = scanRightAsync;
let AsyncScanRightIterable = class AsyncScanRightIterable {
    constructor(source, accumulator, isSeeded, seed) {
        this._source = source;
        this._accumulator = accumulator;
        this._isSeeded = isSeeded;
        this._seed = seed;
    }
    [Symbol.asyncIterator]() {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            const source = yield tslib_1.__await(toArrayAsync_1.toArrayAsync(this._source));
            const accumulator = this._accumulator;
            let hasCurrent = this._isSeeded;
            let current = this._seed;
            for (let offset = source.length - 1; offset >= 0; offset--) {
                const value = source[offset];
                if (!hasCurrent) {
                    current = value;
                    hasCurrent = true;
                    continue;
                }
                current = yield tslib_1.__await(accumulator(current, value, offset));
                yield yield tslib_1.__await(current);
            }
        });
    }
};
AsyncScanRightIterable = tslib_1.__decorate([
    internal_1.ToStringTag("AsyncScanRightIterable")
], AsyncScanRightIterable);
