"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
function skip(source, count) {
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBePositiveFiniteNumber(count, "count");
    return internal_1.FlowHierarchy(new SkipIterable(internal_1.ToIterable(source), count), source);
}
exports.skip = skip;
let SkipIterable = class SkipIterable {
    constructor(source, count) {
        this._source = source;
        this._count = count;
    }
    *[Symbol.iterator]() {
        let remaining = this._count;
        if (remaining <= 0) {
            yield* this._source;
        }
        else {
            for (const element of this._source) {
                if (remaining > 0) {
                    remaining--;
                }
                else {
                    yield element;
                }
            }
        }
    }
};
SkipIterable = tslib_1.__decorate([
    internal_1.ToStringTag("SkipIterable")
], SkipIterable);
