"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const common_1 = require("./common");
/**
 * Computes a scalar value indicating whether `source` contains any elements,
 * optionally filtering the elements using the supplied callback.
 *
 * @param source An [[AsyncQueryable]] object.
 * @param predicate An optional callback used to match each element.
 * @category Scalar
 */
function someAsync(source, predicate = common_1.T) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        var e_1, _a;
        internal_1.assert.mustBeAsyncQueryable(source, "source");
        internal_1.assert.mustBeFunction(predicate, "predicate");
        try {
            for (var _b = tslib_1.__asyncValues(internal_1.ToPossiblyAsyncIterable(source)), _c; _c = yield _b.next(), !_c.done;) {
                const element = _c.value;
                const result = predicate(element);
                if (typeof result === "boolean" ? result : yield result) {
                    return true;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return false;
    });
}
exports.someAsync = someAsync;
