"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const common_1 = require("./common");
function spanMap(source, keySelector, elementSelector = common_1.identity, spanSelector = internal_1.CreateGrouping) {
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    internal_1.assert.mustBeFunction(elementSelector, "elementSelector");
    internal_1.assert.mustBeFunction(spanSelector, "spanSelector");
    return new SpanMapIterable(internal_1.ToIterable(source), keySelector, elementSelector, spanSelector);
}
exports.spanMap = spanMap;
let SpanMapIterable = class SpanMapIterable {
    constructor(source, keySelector, elementSelector, spanSelector) {
        this._source = source;
        this._keySelector = keySelector;
        this._elementSelector = elementSelector;
        this._spanSelector = spanSelector;
    }
    *[Symbol.iterator]() {
        const source = internal_1.ToIterable(this._source);
        const keySelector = this._keySelector;
        const elementSelector = this._elementSelector;
        const spanSelector = this._spanSelector;
        let span;
        let previousKey;
        for (const element of source) {
            const key = keySelector(element);
            if (!span) {
                previousKey = key;
                span = [];
            }
            else if (!internal_1.SameValue(previousKey, key)) {
                yield spanSelector(previousKey, elementSelector === common_1.identity ? internal_1.FlowHierarchy(span, this._source) : span);
                span = [];
                previousKey = key;
            }
            span.push(elementSelector(element));
        }
        if (span) {
            yield spanSelector(previousKey, elementSelector === common_1.identity ? internal_1.FlowHierarchy(span, this._source) : span);
        }
    }
};
SpanMapIterable = tslib_1.__decorate([
    internal_1.ToStringTag("SpanMapIterable")
], SpanMapIterable);
