"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
function startsWithAsync(left, right, equalityComparison = internal_1.SameValue) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        internal_1.assert.mustBeAsyncQueryable(left, "left");
        internal_1.assert.mustBeAsyncQueryable(right, "right");
        internal_1.assert.mustBeFunction(equalityComparison, "equalityComparison");
        const leftIterator = internal_1.GetAsyncIterator(internal_1.ToPossiblyAsyncIterable(left));
        let leftDone = false;
        let leftValue;
        try {
            const rightIterator = internal_1.GetAsyncIterator(internal_1.ToPossiblyAsyncIterable(right));
            let rightDone = false;
            let rightValue;
            try {
                for (;;) {
                    ({ done: leftDone, value: leftValue } = yield leftIterator.next());
                    ({ done: rightDone, value: rightValue } = yield rightIterator.next());
                    if (rightDone)
                        return true;
                    if (leftDone || !equalityComparison(leftValue, rightValue))
                        return false;
                }
            }
            finally {
                if (!rightDone)
                    yield internal_1.AsyncIteratorClose(rightIterator);
            }
        }
        finally {
            if (!leftDone)
                yield internal_1.AsyncIteratorClose(leftIterator);
        }
    });
}
exports.startsWithAsync = startsWithAsync;
