"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const collections_1 = require("../collections");
function symmetricDifferenceByAsync(left, right, keySelector) {
    internal_1.assert.mustBeAsyncQueryable(left, "left");
    internal_1.assert.mustBeAsyncQueryable(right, "right");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    return internal_1.FlowHierarchy(new AsyncSymmetricDifferenceByIterable(internal_1.ToPossiblyAsyncIterable(left), internal_1.ToPossiblyAsyncIterable(right), keySelector), left, right);
}
exports.symmetricDifferenceByAsync = symmetricDifferenceByAsync;
let AsyncSymmetricDifferenceByIterable = class AsyncSymmetricDifferenceByIterable {
    constructor(left, right, keySelector) {
        this._left = left;
        this._right = right;
        this._keySelector = keySelector;
    }
    [Symbol.asyncIterator]() {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            var e_1, _a, e_2, _b;
            const keySelector = this._keySelector;
            const rightKeys = new collections_1.Set();
            const right = new Map();
            try {
                for (var _c = tslib_1.__asyncValues(this._right), _d; _d = yield tslib_1.__await(_c.next()), !_d.done;) {
                    const element = _d.value;
                    const key = keySelector(element);
                    if (internal_1.TryAdd(rightKeys, key)) {
                        right.set(key, element);
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_d && !_d.done && (_a = _c.return)) yield tslib_1.__await(_a.call(_c));
                }
                finally { if (e_1) throw e_1.error; }
            }
            const set = new collections_1.Set();
            try {
                for (var _e = tslib_1.__asyncValues(this._left), _f; _f = yield tslib_1.__await(_e.next()), !_f.done;) {
                    const element = _f.value;
                    const key = keySelector(element);
                    if (internal_1.TryAdd(set, key) && !right.has(key)) {
                        yield yield tslib_1.__await(element);
                    }
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (_f && !_f.done && (_b = _e.return)) yield tslib_1.__await(_b.call(_e));
                }
                finally { if (e_2) throw e_2.error; }
            }
            for (const [key, element] of right) {
                if (internal_1.TryAdd(set, key)) {
                    yield yield tslib_1.__await(element);
                }
            }
        });
    }
};
AsyncSymmetricDifferenceByIterable = tslib_1.__decorate([
    internal_1.ToStringTag("AsyncSymmetricDifferenceByIterable")
], AsyncSymmetricDifferenceByIterable);
