/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
import { Queryable } from "../types";
/**
 * Creates a subquery that combines two [[Queryable]] objects by combining elements
 * in tuples.
 *
 * @param left A [[Queryable]].
 * @param right A [[Queryable]].
 * @category Join
 */
export declare function zip<T, U>(left: Queryable<T>, right: Queryable<U>): Iterable<[T, U]>;
/**
 * Creates a subquery that combines two [[Queryable]] objects by combining elements
 * using the supplied callback.
 *
 * @param left A [[Queryable]].
 * @param right A [[Queryable]].
 * @param selector A callback used to combine two elements.
 * @category Join
 */
export declare function zip<T, U, R>(left: Queryable<T>, right: Queryable<U>, selector: (left: T, right: U) => R): Iterable<R>;
