"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var collections_1 = require("../collections");
/**
 * Creates an [[Iterable]] that iterates the elements from sources picked from a list based on the
 * result of a lazily evaluated choice.
 *
 * @param chooser A callback used to choose a source.
 * @param choices A [[Queryable]] of key/value pairs, where each value is a [[Queryable]] object.
 * @param otherwise A default source to use when another choice could not be made.
 * @category Query
 */
function choose(chooser, choices, otherwise) {
    internal_1.assert.mustBeFunction(chooser, "chooser");
    internal_1.assert.mustBeQueryable(choices, "choices");
    internal_1.assert.mustBeQueryableOrUndefined(otherwise, "otherwise");
    return new ChooseIterable(chooser, internal_1.ToIterable(choices), otherwise && internal_1.ToIterable(otherwise));
}
exports.choose = choose;
var ChooseIterable = /** @class */ (function () {
    function ChooseIterable(chooser, choices, otherwise) {
        this._chooser = chooser;
        this._choices = choices;
        this._otherwise = otherwise;
    }
    ChooseIterable.prototype[Symbol.iterator] = function () {
        var chooser, choices, otherwise, choice, result;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    chooser = this._chooser;
                    choices = new collections_1.Map(this._choices);
                    otherwise = this._otherwise;
                    choice = chooser();
                    result = choices.get(choice);
                    if (!(result !== undefined && result !== null)) return [3 /*break*/, 2];
                    return [5 /*yield**/, tslib_1.__values(internal_1.ToIterable(result))];
                case 1:
                    _a.sent();
                    return [3 /*break*/, 4];
                case 2:
                    if (!otherwise) return [3 /*break*/, 4];
                    return [5 /*yield**/, tslib_1.__values(otherwise)];
                case 3:
                    _a.sent();
                    _a.label = 4;
                case 4: return [2 /*return*/];
            }
        });
    };
    ChooseIterable = tslib_1.__decorate([
        internal_1.ToStringTag("ChooseIterable")
    ], ChooseIterable);
    return ChooseIterable;
}());
