"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
/**
 * Writes each element of a source iterable to a destination array.
 *
 * @param source A [[Queryable]] object.
 * @param dest The destination array.
 * @param start The offset into the array at which to start writing.
 * @param count The number of elements to write to the array.
 * @category Scalar
 */
function copyToAsync(source, dest, start, count) {
    if (start === void 0) { start = 0; }
    if (count === void 0) { count = dest.length - start; }
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var e_1, _a, minLength, _b, _c, element, e_1_1;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    internal_1.assert.mustBeAsyncQueryable(source, "source");
                    internal_1.assert.mustBeArrayLike(dest, "dest");
                    internal_1.assert.mustBePositiveInteger(start, "start");
                    internal_1.assert.mustBePositiveInteger(count, "count");
                    if (!(count > 0)) return [3 /*break*/, 13];
                    minLength = start + count;
                    _d.label = 1;
                case 1:
                    _d.trys.push([1, 6, 7, 12]);
                    _b = tslib_1.__asyncValues(internal_1.ToPossiblyAsyncIterable(source));
                    _d.label = 2;
                case 2: return [4 /*yield*/, _b.next()];
                case 3:
                    if (!(_c = _d.sent(), !_c.done)) return [3 /*break*/, 5];
                    element = _c.value;
                    if (count > 0) {
                        dest[start++] = element;
                        count--;
                    }
                    _d.label = 4;
                case 4: return [3 /*break*/, 2];
                case 5: return [3 /*break*/, 12];
                case 6:
                    e_1_1 = _d.sent();
                    e_1 = { error: e_1_1 };
                    return [3 /*break*/, 12];
                case 7:
                    _d.trys.push([7, , 10, 11]);
                    if (!(_c && !_c.done && (_a = _b.return))) return [3 /*break*/, 9];
                    return [4 /*yield*/, _a.call(_b)];
                case 8:
                    _d.sent();
                    _d.label = 9;
                case 9: return [3 /*break*/, 11];
                case 10:
                    if (e_1) throw e_1.error;
                    return [7 /*endfinally*/];
                case 11: return [7 /*endfinally*/];
                case 12:
                    if (dest.length < minLength)
                        dest.length = minLength;
                    _d.label = 13;
                case 13: return [2 /*return*/, dest];
            }
        });
    });
}
exports.copyToAsync = copyToAsync;
