"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var collections_1 = require("../collections");
var common_1 = require("./common");
/**
 * Counts the number of elements, optionally filtering elements using the supplied callback.
 *
 * @param source A [[Queryable]] object.
 * @param predicate An optional callback used to match each element.
 * @category Scalar
 */
function countAsync(source, predicate) {
    if (predicate === void 0) { predicate = common_1.T; }
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var e_1, _a, realSource, count, _b, _c, element, _d, e_1_1;
        return tslib_1.__generator(this, function (_e) {
            switch (_e.label) {
                case 0:
                    internal_1.assert.mustBeAsyncQueryable(source, "source");
                    internal_1.assert.mustBeFunction(predicate, "predicate");
                    if (predicate === common_1.T) {
                        realSource = internal_1.GetAsyncSource(source);
                        if (Array.isArray(realSource))
                            return [2 /*return*/, realSource.length];
                        if (realSource instanceof collections_1.Set || realSource instanceof collections_1.Map)
                            return [2 /*return*/, realSource.size];
                    }
                    count = 0;
                    _e.label = 1;
                case 1:
                    _e.trys.push([1, 8, 9, 14]);
                    _b = tslib_1.__asyncValues(internal_1.ToPossiblyAsyncIterable(source));
                    _e.label = 2;
                case 2: return [4 /*yield*/, _b.next()];
                case 3:
                    if (!(_c = _e.sent(), !_c.done)) return [3 /*break*/, 7];
                    element = _c.value;
                    _d = predicate === common_1.T;
                    if (_d) return [3 /*break*/, 5];
                    return [4 /*yield*/, predicate(element)];
                case 4:
                    _d = (_e.sent());
                    _e.label = 5;
                case 5:
                    if (_d) {
                        count++;
                    }
                    _e.label = 6;
                case 6: return [3 /*break*/, 2];
                case 7: return [3 /*break*/, 14];
                case 8:
                    e_1_1 = _e.sent();
                    e_1 = { error: e_1_1 };
                    return [3 /*break*/, 14];
                case 9:
                    _e.trys.push([9, , 12, 13]);
                    if (!(_c && !_c.done && (_a = _b.return))) return [3 /*break*/, 11];
                    return [4 /*yield*/, _a.call(_b)];
                case 10:
                    _e.sent();
                    _e.label = 11;
                case 11: return [3 /*break*/, 13];
                case 12:
                    if (e_1) throw e_1.error;
                    return [7 /*endfinally*/];
                case 13: return [7 /*endfinally*/];
                case 14: return [2 /*return*/, count];
            }
        });
    });
}
exports.countAsync = countAsync;
