"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var collections_1 = require("../collections");
var common_1 = require("./common");
function distinct(source, keySelector) {
    if (keySelector === void 0) { keySelector = common_1.identity; }
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    return internal_1.FlowHierarchy(new DistinctIterable(internal_1.ToIterable(source), keySelector), source);
}
exports.distinct = distinct;
var DistinctIterable = /** @class */ (function () {
    function DistinctIterable(source, keySelector) {
        this._source = source;
        this._keySelector = keySelector;
    }
    DistinctIterable.prototype[Symbol.iterator] = function () {
        var e_1, _a, set, keySelector, _b, _c, element, key, e_1_1;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    set = new collections_1.Set();
                    keySelector = this._keySelector;
                    _d.label = 1;
                case 1:
                    _d.trys.push([1, 6, 7, 8]);
                    _b = tslib_1.__values(this._source), _c = _b.next();
                    _d.label = 2;
                case 2:
                    if (!!_c.done) return [3 /*break*/, 5];
                    element = _c.value;
                    key = keySelector(element);
                    if (!!set.has(key)) return [3 /*break*/, 4];
                    set.add(key);
                    return [4 /*yield*/, element];
                case 3:
                    _d.sent();
                    _d.label = 4;
                case 4:
                    _c = _b.next();
                    return [3 /*break*/, 2];
                case 5: return [3 /*break*/, 8];
                case 6:
                    e_1_1 = _d.sent();
                    e_1 = { error: e_1_1 };
                    return [3 /*break*/, 8];
                case 7:
                    try {
                        if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                    }
                    finally { if (e_1) throw e_1.error; }
                    return [7 /*endfinally*/];
                case 8: return [2 /*return*/];
            }
        });
    };
    DistinctIterable = tslib_1.__decorate([
        internal_1.ToStringTag("DistinctIterable")
    ], DistinctIterable);
    return DistinctIterable;
}());
