"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var last_1 = require("./last");
/**
 * Finds the value at the provided offset. A negative offset starts from the
 * last element.
 *
 * @param source A [[Queryable]] object.
 * @param offset An offset.
 * @category Scalar
 */
function elementAt(source, offset) {
    var e_1, _a, e_2, _b;
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeInteger(offset, "offset");
    if (offset === -1) {
        return last_1.last(source);
    }
    if (offset < 0) {
        offset = Math.abs(offset);
        var array = [];
        try {
            for (var _c = tslib_1.__values(internal_1.ToIterable(source)), _d = _c.next(); !_d.done; _d = _c.next()) {
                var element = _d.value;
                if (array.length >= offset) {
                    array.shift();
                }
                array.push(element);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return array.length - offset >= 0 ? array[array.length - offset] : undefined;
    }
    try {
        for (var _e = tslib_1.__values(internal_1.ToIterable(source)), _f = _e.next(); !_f.done; _f = _e.next()) {
            var element = _f.value;
            if (offset === 0) {
                return element;
            }
            offset--;
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (_f && !_f.done && (_b = _e.return)) _b.call(_e);
        }
        finally { if (e_2) throw e_2.error; }
    }
    return undefined;
}
exports.elementAt = elementAt;
