"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var internal_1 = require("../internal");
var takeRight_1 = require("./takeRight");
var toArray_1 = require("./toArray");
function endsWith(left, right, equalityComparison) {
    if (equalityComparison === void 0) { equalityComparison = internal_1.SameValue; }
    internal_1.assert.mustBeQueryable(left, "left");
    internal_1.assert.mustBeQueryable(right, "right");
    internal_1.assert.mustBeFunction(equalityComparison, "equalityComparison");
    var rightArray = toArray_1.toArray(right);
    var numElements = rightArray.length;
    if (numElements <= 0) {
        return true;
    }
    var leftArray = toArray_1.toArray(takeRight_1.takeRight(left, numElements));
    if (leftArray.length < numElements) {
        return false;
    }
    for (var i = 0; i < numElements; i++) {
        if (!equalityComparison(leftArray[i], rightArray[i])) {
            return false;
        }
    }
    return true;
}
exports.endsWith = endsWith;
