"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
function filterByAsync(source, keySelector, predicate) {
    internal_1.assert.mustBeAsyncQueryable(source, "source");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    internal_1.assert.mustBeFunction(predicate, "predicate");
    return internal_1.FlowHierarchy(new AsyncFilterByIterable(internal_1.ToPossiblyAsyncIterable(source), keySelector, predicate), source);
}
exports.filterByAsync = filterByAsync;
var AsyncFilterByIterable = /** @class */ (function () {
    function AsyncFilterByIterable(source, keySelector, predicate) {
        this._source = source;
        this._keySelector = keySelector;
        this._predicate = predicate;
    }
    AsyncFilterByIterable.prototype[Symbol.asyncIterator] = function () {
        return tslib_1.__asyncGenerator(this, arguments, function _a() {
            var e_1, _a, keySelector, predicate, offset, _b, _c, element, result, _d, e_1_1;
            return tslib_1.__generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        keySelector = this._keySelector;
                        predicate = this._predicate;
                        offset = 0;
                        _e.label = 1;
                    case 1:
                        _e.trys.push([1, 11, 12, 17]);
                        _b = tslib_1.__asyncValues(this._source);
                        _e.label = 2;
                    case 2: return [4 /*yield*/, tslib_1.__await(_b.next())];
                    case 3:
                        if (!(_c = _e.sent(), !_c.done)) return [3 /*break*/, 10];
                        element = _c.value;
                        result = predicate(keySelector(element), offset++);
                        if (!(typeof result === "boolean")) return [3 /*break*/, 4];
                        _d = result;
                        return [3 /*break*/, 6];
                    case 4: return [4 /*yield*/, tslib_1.__await(result)];
                    case 5:
                        _d = _e.sent();
                        _e.label = 6;
                    case 6:
                        if (!_d) return [3 /*break*/, 9];
                        return [4 /*yield*/, tslib_1.__await(element)];
                    case 7: return [4 /*yield*/, _e.sent()];
                    case 8:
                        _e.sent();
                        _e.label = 9;
                    case 9: return [3 /*break*/, 2];
                    case 10: return [3 /*break*/, 17];
                    case 11:
                        e_1_1 = _e.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 17];
                    case 12:
                        _e.trys.push([12, , 15, 16]);
                        if (!(_c && !_c.done && (_a = _b.return))) return [3 /*break*/, 14];
                        return [4 /*yield*/, tslib_1.__await(_a.call(_b))];
                    case 13:
                        _e.sent();
                        _e.label = 14;
                    case 14: return [3 /*break*/, 16];
                    case 15:
                        if (e_1) throw e_1.error;
                        return [7 /*endfinally*/];
                    case 16: return [7 /*endfinally*/];
                    case 17: return [2 /*return*/];
                }
            });
        });
    };
    AsyncFilterByIterable = tslib_1.__decorate([
        internal_1.ToStringTag("AsyncFilterByIterable")
    ], AsyncFilterByIterable);
    return AsyncFilterByIterable;
}());
