"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
/**
 * Creates an [[AsyncIterable]] whose values are provided by a callback executed a provided number of
 * times.
 *
 * @param count The number of times to execute the callback.
 * @param generator The callback to execute.
 * @category Query
 */
function generateAsync(count, generator) {
    internal_1.assert.mustBePositiveFiniteNumber(count, "count");
    internal_1.assert.mustBeFunction(generator, "generator");
    return new AsyncGenerateIterable(count, generator);
}
exports.generateAsync = generateAsync;
var AsyncGenerateIterable = /** @class */ (function () {
    function AsyncGenerateIterable(count, generator) {
        this._count = count;
        this._generator = generator;
    }
    AsyncGenerateIterable.prototype[Symbol.asyncIterator] = function () {
        return tslib_1.__asyncGenerator(this, arguments, function _a() {
            var count, generator, i;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        count = this._count;
                        generator = this._generator;
                        i = 0;
                        _a.label = 1;
                    case 1:
                        if (!(i < count)) return [3 /*break*/, 5];
                        return [4 /*yield*/, tslib_1.__await(generator(i))];
                    case 2: return [4 /*yield*/, _a.sent()];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4:
                        i++;
                        return [3 /*break*/, 1];
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    AsyncGenerateIterable = tslib_1.__decorate([
        internal_1.ToStringTag("AsyncGenerateIterable")
    ], AsyncGenerateIterable);
    return AsyncGenerateIterable;
}());
