"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var common_1 = require("./common");
function groupByAsync(source, keySelector, elementSelector, resultSelector) {
    if (elementSelector === void 0) { elementSelector = common_1.identity; }
    if (resultSelector === void 0) { resultSelector = internal_1.CreateGrouping; }
    internal_1.assert.mustBeAsyncQueryable(source, "source");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    internal_1.assert.mustBeFunction(elementSelector, "elementSelector");
    internal_1.assert.mustBeFunction(resultSelector, "resultSelector");
    return new AsyncGroupByIterable(internal_1.ToPossiblyAsyncIterable(source), keySelector, elementSelector, resultSelector);
}
exports.groupByAsync = groupByAsync;
var AsyncGroupByIterable = /** @class */ (function () {
    function AsyncGroupByIterable(source, keySelector, elementSelector, resultSelector) {
        this._source = source;
        this._keySelector = keySelector;
        this._elementSelector = elementSelector;
        this._resultSelector = resultSelector;
    }
    AsyncGroupByIterable.prototype[Symbol.asyncIterator] = function () {
        return tslib_1.__asyncGenerator(this, arguments, function _a() {
            var e_1, _a, source, elementSelector, resultSelector, map, map_1, map_1_1, _b, key, values, e_1_1;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        source = this._source;
                        elementSelector = this._elementSelector;
                        resultSelector = this._resultSelector;
                        return [4 /*yield*/, tslib_1.__await(internal_1.CreateGroupingsAsync(source, this._keySelector, this._elementSelector))];
                    case 1:
                        map = _c.sent();
                        _c.label = 2;
                    case 2:
                        _c.trys.push([2, 8, 9, 10]);
                        map_1 = tslib_1.__values(map), map_1_1 = map_1.next();
                        _c.label = 3;
                    case 3:
                        if (!!map_1_1.done) return [3 /*break*/, 7];
                        _b = tslib_1.__read(map_1_1.value, 2), key = _b[0], values = _b[1];
                        return [4 /*yield*/, tslib_1.__await(resultSelector(key, elementSelector === common_1.identity ? internal_1.FlowHierarchy(values, source) : values))];
                    case 4: return [4 /*yield*/, _c.sent()];
                    case 5:
                        _c.sent();
                        _c.label = 6;
                    case 6:
                        map_1_1 = map_1.next();
                        return [3 /*break*/, 3];
                    case 7: return [3 /*break*/, 10];
                    case 8:
                        e_1_1 = _c.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 10];
                    case 9:
                        try {
                            if (map_1_1 && !map_1_1.done && (_a = map_1.return)) _a.call(map_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                        return [7 /*endfinally*/];
                    case 10: return [2 /*return*/];
                }
            });
        });
    };
    AsyncGroupByIterable = tslib_1.__decorate([
        internal_1.ToStringTag("AsyncGroupByIterable")
    ], AsyncGroupByIterable);
    return AsyncGroupByIterable;
}());
