"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var toArray_1 = require("./toArray");
function includesSequence(left, right, equalityComparison) {
    if (equalityComparison === void 0) { equalityComparison = internal_1.SameValue; }
    var e_1, _a;
    internal_1.assert.mustBeQueryable(left, "source");
    internal_1.assert.mustBeQueryable(right, "other");
    internal_1.assert.mustBeFunction(equalityComparison, "equalityComparison");
    var rightArray = toArray_1.toArray(right);
    var numRightElements = rightArray.length;
    if (numRightElements <= 0) {
        return true;
    }
    var span = [];
    try {
        for (var _b = tslib_1.__values(internal_1.ToIterable(left)), _c = _b.next(); !_c.done; _c = _b.next()) {
            var leftValue = _c.value;
            for (;;) {
                var rightValue = rightArray[span.length];
                if (equalityComparison(leftValue, rightValue)) {
                    if (span.length + 1 >= numRightElements) {
                        return true;
                    }
                    span.push(leftValue);
                }
                else if (span.length > 0) {
                    span.shift();
                    continue;
                }
                break;
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return false;
}
exports.includesSequence = includesSequence;
