"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var toSetAsync_1 = require("./toSetAsync");
function intersectByAsync(left, right, keySelector) {
    internal_1.assert.mustBeAsyncQueryable(left, "left");
    internal_1.assert.mustBeAsyncQueryable(right, "right");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    return internal_1.FlowHierarchy(new AsyncIntersectByIterable(internal_1.ToPossiblyAsyncIterable(left), internal_1.ToPossiblyAsyncIterable(right), keySelector), left, right);
}
exports.intersectByAsync = intersectByAsync;
var AsyncIntersectByIterable = /** @class */ (function () {
    function AsyncIntersectByIterable(left, right, keySelector) {
        this._left = left;
        this._right = right;
        this._keySelector = keySelector;
    }
    AsyncIntersectByIterable.prototype[Symbol.asyncIterator] = function () {
        return tslib_1.__asyncGenerator(this, arguments, function _a() {
            var e_1, _a, keySelector, set, _b, _c, element, e_1_1;
            return tslib_1.__generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        keySelector = this._keySelector;
                        return [4 /*yield*/, tslib_1.__await(toSetAsync_1.toSetAsync(this._right, keySelector))];
                    case 1:
                        set = _d.sent();
                        if (!(set.size <= 0)) return [3 /*break*/, 3];
                        return [4 /*yield*/, tslib_1.__await(void 0)];
                    case 2: return [2 /*return*/, _d.sent()];
                    case 3:
                        _d.trys.push([3, 10, 11, 16]);
                        _b = tslib_1.__asyncValues(this._left);
                        _d.label = 4;
                    case 4: return [4 /*yield*/, tslib_1.__await(_b.next())];
                    case 5:
                        if (!(_c = _d.sent(), !_c.done)) return [3 /*break*/, 9];
                        element = _c.value;
                        if (!set.delete(keySelector(element))) return [3 /*break*/, 8];
                        return [4 /*yield*/, tslib_1.__await(element)];
                    case 6: return [4 /*yield*/, _d.sent()];
                    case 7:
                        _d.sent();
                        _d.label = 8;
                    case 8: return [3 /*break*/, 4];
                    case 9: return [3 /*break*/, 16];
                    case 10:
                        e_1_1 = _d.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 16];
                    case 11:
                        _d.trys.push([11, , 14, 15]);
                        if (!(_c && !_c.done && (_a = _b.return))) return [3 /*break*/, 13];
                        return [4 /*yield*/, tslib_1.__await(_a.call(_b))];
                    case 12:
                        _d.sent();
                        _d.label = 13;
                    case 13: return [3 /*break*/, 15];
                    case 14:
                        if (e_1) throw e_1.error;
                        return [7 /*endfinally*/];
                    case 15: return [7 /*endfinally*/];
                    case 16: return [2 /*return*/];
                }
            });
        });
    };
    AsyncIntersectByIterable = tslib_1.__decorate([
        internal_1.ToStringTag("AsyncIntersectByIterable")
    ], AsyncIntersectByIterable);
    return AsyncIntersectByIterable;
}());
