"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var internal_1 = require("../internal");
var common_1 = require("./common");
var maxBy_1 = require("./maxBy");
/**
 * Gets the maximum element of a [[Queryable]], optionally comparing elements using the supplied callback.
 *
 * @param source A [[Queryable]] object.
 * @param comparison An optional callback used to compare two elements.
 * @category Scalar
 */
function max(source, comparison) {
    if (comparison === void 0) { comparison = common_1.compare; }
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeFunction(comparison, "comparison");
    return maxBy_1.maxBy(source, common_1.identity, comparison);
}
exports.max = max;
