"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var types_1 = require("../types");
var elementAt_1 = require("./elementAt");
var axis_1 = require("./axis");
/**
 * Creates a [[HierarchyIterable]] for the child of each element at the specified offset. A negative offset
 * starts from the last child.
 *
 * @param source A [[HierarchyIterable]] object.
 * @param offset The offset for the child.
 * @category Hierarchy
 */
function nthChild(source, offset) {
    internal_1.assert.mustBeHierarchyIterable(source, "source");
    internal_1.assert.mustBeInteger(offset, "offset");
    return new NthChildIterable(source, offset);
}
exports.nthChild = nthChild;
function nthChildOf(element, provider, offset) {
    if (element !== undefined) {
        var children = axis_1.Axis.children(provider, element);
        return elementAt_1.elementAt(children, offset);
    }
    return undefined;
}
var NthChildIterable = /** @class */ (function () {
    function NthChildIterable(source, offset) {
        this._source = source;
        this._offset = offset;
    }
    NthChildIterable.prototype[Symbol.iterator] = function () {
        var e_1, _a, source, hierarchy, offset, source_1, source_1_1, element, child, e_1_1;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    source = this._source;
                    hierarchy = internal_1.GetHierarchy(source);
                    offset = this._offset;
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 6, 7, 8]);
                    source_1 = tslib_1.__values(source), source_1_1 = source_1.next();
                    _b.label = 2;
                case 2:
                    if (!!source_1_1.done) return [3 /*break*/, 5];
                    element = source_1_1.value;
                    child = nthChildOf(element, hierarchy, offset);
                    if (!(child !== undefined)) return [3 /*break*/, 4];
                    return [4 /*yield*/, child];
                case 3:
                    _b.sent();
                    _b.label = 4;
                case 4:
                    source_1_1 = source_1.next();
                    return [3 /*break*/, 2];
                case 5: return [3 /*break*/, 8];
                case 6:
                    e_1_1 = _b.sent();
                    e_1 = { error: e_1_1 };
                    return [3 /*break*/, 8];
                case 7:
                    try {
                        if (source_1_1 && !source_1_1.done && (_a = source_1.return)) _a.call(source_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                    return [7 /*endfinally*/];
                case 8: return [2 /*return*/];
            }
        });
    };
    NthChildIterable.prototype[types_1.Hierarchical.hierarchy] = function () {
        return internal_1.GetHierarchy(this._source);
    };
    NthChildIterable = tslib_1.__decorate([
        internal_1.ToStringTag("NthChildIterable")
    ], NthChildIterable);
    return NthChildIterable;
}());
