/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
import { PossiblyAsyncHierarchyIterable, AsyncOrderedHierarchyIterable, AsyncQueryable, AsyncOrderedIterable, PossiblyAsyncOrderedHierarchyIterable, PossiblyAsyncOrderedIterable } from "../types";
/**
 * Creates an [[AsyncOrderedHierarchyIterable]] whose elements are sorted in ascending order by the provided key.
 *
 * @param source A [[HierarchyIterable]] or [[AsyncHierarchyIterable]] object.
 * @param keySelector A callback used to select the key for an element.
 * @param comparison An optional callback used to compare two keys.
 * @category Order
 */
export declare function orderByAsync<TNode, T extends TNode, K>(source: PossiblyAsyncHierarchyIterable<TNode, T>, keySelector: (element: T) => K, comparison?: (x: K, y: K) => number): AsyncOrderedHierarchyIterable<TNode, T>;
/**
 * Creates an [[AsyncOrderedIterable]] whose elements are sorted in ascending order by the provided key.
 *
 * @param source An [[AsyncQueryable]] object.
 * @param keySelector A callback used to select the key for an element.
 * @param comparison An optional callback used to compare two keys.
 * @category Order
 */
export declare function orderByAsync<T, K>(source: AsyncQueryable<T>, keySelector: (element: T) => K, comparison?: (x: K, y: K) => number): AsyncOrderedIterable<T>;
/**
 * Creates an [[AsyncOrderedHierarchyIterable]] whose elements are sorted in descending order by the provided key.
 *
 * @param source A [[HierarchyIterable]] or [[AsyncHierarchyIterable]] object.
 * @param keySelector A callback used to select the key for an element.
 * @param comparison An optional callback used to compare two keys.
 * @category Order
 */
export declare function orderByDescendingAsync<TNode, T extends TNode, K>(source: PossiblyAsyncHierarchyIterable<TNode, T>, keySelector: (element: T) => K, comparison?: (x: K, y: K) => number): AsyncOrderedHierarchyIterable<TNode, T>;
/**
 * Creates an [[AsyncOrderedIterable]] whose elements are sorted in descending order by the provided key.
 *
 * @param source An [[AsyncQueryable]] object.
 * @param keySelector A callback used to select the key for an element.
 * @param comparison An optional callback used to compare two keys.
 * @category Order
 */
export declare function orderByDescendingAsync<T, K>(source: AsyncQueryable<T>, keySelector: (element: T) => K, comparison?: (x: K, y: K) => number): AsyncOrderedIterable<T>;
/**
 * Creates a subsequent [[AsyncOrderedHierarchyIterable]] whose elements are also sorted in ascending order by the provided key.
 *
 * @param source An [[OrderedHierarchyIterable]] or [[AsyncOrderedHierarchyIterable]] object.
 * @param keySelector A callback used to select the key for an element.
 * @param comparison An optional callback used to compare two keys.
 * @category Order
 */
export declare function thenByAsync<TNode, T extends TNode, K>(source: PossiblyAsyncOrderedHierarchyIterable<TNode, T>, keySelector: (element: T) => K, comparison?: (x: K, y: K) => number): AsyncOrderedHierarchyIterable<TNode, T>;
/**
 * Creates a subsequent [[AsyncOrderedIterable]] whose elements are also sorted in ascending order by the provided key.
 *
 * @param source An [[OrderedIterable]] or [[AsyncOrderedIterable]] object.
 * @param keySelector A callback used to select the key for an element.
 * @param comparison An optional callback used to compare two keys.
 * @category Order
 */
export declare function thenByAsync<T, K>(source: PossiblyAsyncOrderedIterable<T>, keySelector: (element: T) => K, comparison?: (x: K, y: K) => number): AsyncOrderedIterable<T>;
/**
 * Creates a subsequent [[AsyncOrderedHierarchyIterable]] whose elements are also sorted in descending order by the provided key.
 *
 * @param source An [[OrderedHierarchyIterable]] or [[AsyncOrderedHierarchyIterable]] object.
 * @param keySelector A callback used to select the key for an element.
 * @param comparison An optional callback used to compare two keys.
 * @category Order
 */
export declare function thenByDescendingAsync<TNode, T extends TNode, K>(source: PossiblyAsyncOrderedHierarchyIterable<TNode, T>, keySelector: (element: T) => K, comparison?: (x: K, y: K) => number): AsyncOrderedHierarchyIterable<TNode, T>;
/**
 * Creates a subsequent [[AsyncOrderedIterable]] whose elements are also sorted in descending order by the provided key.
 *
 * @param source An [[OrderedIterable]] or [[AsyncOrderedIterable]] object.
 * @param keySelector A callback used to select the key for an element.
 * @param comparison An optional callback used to compare two keys.
 * @category Order
 */
export declare function thenByDescendingAsync<T, K>(source: PossiblyAsyncOrderedIterable<T>, keySelector: (element: T) => K, comparison?: (x: K, y: K) => number): AsyncOrderedIterable<T>;
