/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
import { Queryable, Page, HierarchyIterable, HierarchyPage } from "../types";
/**
 * Creates a [[HierarchyIterable]] that splits a [[Queryable]] into one or more pages.
 * While advancing from page to page is evaluated lazily, the elements of the page are
 * evaluated eagerly.
 *
 * @param source A [[HierarchyIterable]] object.
 * @param pageSize The number of elements per page.
 * @category Subquery
 */
export declare function pageBy<TNode, T extends TNode>(source: HierarchyIterable<TNode, T>, pageSize: number): Iterable<HierarchyPage<TNode, T>>;
/**
 * Creates an [[Iterable]] that splits a [[Queryable]] into one or more pages.
 * While advancing from page to page is evaluated lazily, the elements of the page are
 * evaluated eagerly.
 *
 * @param source A [[Queryable]] object.
 * @param pageSize The number of elements per page.
 * @category Subquery
 */
export declare function pageBy<T>(source: Queryable<T>, pageSize: number): Iterable<Page<T>>;
