"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
function patch(source, start, skipCount, range) {
    if (skipCount === void 0) { skipCount = 0; }
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBePositiveFiniteNumber(start, "start");
    internal_1.assert.mustBePositiveFiniteNumber(skipCount, "skipCount");
    internal_1.assert.mustBeQueryableOrUndefined(range, "range");
    return internal_1.FlowHierarchy(new PatchIterable(internal_1.ToIterable(source), start, skipCount, range && internal_1.ToIterable(range)), source);
}
exports.patch = patch;
var PatchIterable = /** @class */ (function () {
    function PatchIterable(source, start, skipCount, range) {
        this._source = source;
        this._start = start;
        this._skipCount = skipCount;
        this._range = range;
    }
    PatchIterable.prototype[Symbol.iterator] = function () {
        var e_1, _a, start, skipCount, offset, hasYieldedRange, _b, _c, value, e_1_1;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    start = this._start;
                    skipCount = this._skipCount;
                    offset = 0;
                    hasYieldedRange = false;
                    _d.label = 1;
                case 1:
                    _d.trys.push([1, 11, 12, 13]);
                    _b = tslib_1.__values(this._source), _c = _b.next();
                    _d.label = 2;
                case 2:
                    if (!!_c.done) return [3 /*break*/, 10];
                    value = _c.value;
                    if (!(offset < start)) return [3 /*break*/, 4];
                    return [4 /*yield*/, value];
                case 3:
                    _d.sent();
                    offset++;
                    return [3 /*break*/, 9];
                case 4:
                    if (!(offset < start + skipCount)) return [3 /*break*/, 5];
                    offset++;
                    return [3 /*break*/, 9];
                case 5:
                    if (!(!hasYieldedRange && this._range)) return [3 /*break*/, 7];
                    return [5 /*yield**/, tslib_1.__values(this._range)];
                case 6:
                    _d.sent();
                    hasYieldedRange = true;
                    _d.label = 7;
                case 7: return [4 /*yield*/, value];
                case 8:
                    _d.sent();
                    _d.label = 9;
                case 9:
                    _c = _b.next();
                    return [3 /*break*/, 2];
                case 10: return [3 /*break*/, 13];
                case 11:
                    e_1_1 = _d.sent();
                    e_1 = { error: e_1_1 };
                    return [3 /*break*/, 13];
                case 12:
                    try {
                        if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                    }
                    finally { if (e_1) throw e_1.error; }
                    return [7 /*endfinally*/];
                case 13:
                    if (!(!hasYieldedRange && this._range)) return [3 /*break*/, 15];
                    return [5 /*yield**/, tslib_1.__values(this._range)];
                case 14:
                    _d.sent();
                    _d.label = 15;
                case 15: return [2 /*return*/];
            }
        });
    };
    PatchIterable = tslib_1.__decorate([
        internal_1.ToStringTag("PatchIterable")
    ], PatchIterable);
    return PatchIterable;
}());
