"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
function patchAsync(source, start, skipCount, range) {
    if (skipCount === void 0) { skipCount = 0; }
    internal_1.assert.mustBeAsyncQueryable(source, "source");
    internal_1.assert.mustBePositiveFiniteNumber(start, "start");
    internal_1.assert.mustBePositiveFiniteNumber(skipCount, "skipCount");
    internal_1.assert.mustBeAsyncQueryableOrUndefined(range, "range");
    return internal_1.FlowHierarchy(new AsyncPatchIterable(internal_1.ToPossiblyAsyncIterable(source), start, skipCount, range && internal_1.ToPossiblyAsyncIterable(range)), source);
}
exports.patchAsync = patchAsync;
var AsyncPatchIterable = /** @class */ (function () {
    function AsyncPatchIterable(source, start, skipCount, range) {
        this._source = source;
        this._start = start;
        this._skipCount = skipCount;
        this._range = range;
    }
    AsyncPatchIterable.prototype[Symbol.asyncIterator] = function () {
        return tslib_1.__asyncGenerator(this, arguments, function _a() {
            var e_1, _a, start, skipCount, offset, hasYieldedRange, _b, _c, value, e_1_1;
            return tslib_1.__generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        start = this._start;
                        skipCount = this._skipCount;
                        offset = 0;
                        hasYieldedRange = false;
                        _d.label = 1;
                    case 1:
                        _d.trys.push([1, 15, 16, 21]);
                        _b = tslib_1.__asyncValues(this._source);
                        _d.label = 2;
                    case 2: return [4 /*yield*/, tslib_1.__await(_b.next())];
                    case 3:
                        if (!(_c = _d.sent(), !_c.done)) return [3 /*break*/, 14];
                        value = _c.value;
                        if (!(offset < start)) return [3 /*break*/, 6];
                        return [4 /*yield*/, tslib_1.__await(value)];
                    case 4: return [4 /*yield*/, _d.sent()];
                    case 5:
                        _d.sent();
                        offset++;
                        return [3 /*break*/, 13];
                    case 6:
                        if (!(offset < start + skipCount)) return [3 /*break*/, 7];
                        offset++;
                        return [3 /*break*/, 13];
                    case 7:
                        if (!(!hasYieldedRange && this._range)) return [3 /*break*/, 10];
                        return [5 /*yield**/, tslib_1.__values(tslib_1.__asyncDelegator(tslib_1.__asyncValues(this._range)))];
                    case 8: return [4 /*yield*/, tslib_1.__await.apply(void 0, [_d.sent()])];
                    case 9:
                        _d.sent();
                        hasYieldedRange = true;
                        _d.label = 10;
                    case 10: return [4 /*yield*/, tslib_1.__await(value)];
                    case 11: return [4 /*yield*/, _d.sent()];
                    case 12:
                        _d.sent();
                        _d.label = 13;
                    case 13: return [3 /*break*/, 2];
                    case 14: return [3 /*break*/, 21];
                    case 15:
                        e_1_1 = _d.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 21];
                    case 16:
                        _d.trys.push([16, , 19, 20]);
                        if (!(_c && !_c.done && (_a = _b.return))) return [3 /*break*/, 18];
                        return [4 /*yield*/, tslib_1.__await(_a.call(_b))];
                    case 17:
                        _d.sent();
                        _d.label = 18;
                    case 18: return [3 /*break*/, 20];
                    case 19:
                        if (e_1) throw e_1.error;
                        return [7 /*endfinally*/];
                    case 20: return [7 /*endfinally*/];
                    case 21:
                        if (!(!hasYieldedRange && this._range)) return [3 /*break*/, 24];
                        return [5 /*yield**/, tslib_1.__values(tslib_1.__asyncDelegator(tslib_1.__asyncValues(this._range)))];
                    case 22: return [4 /*yield*/, tslib_1.__await.apply(void 0, [_d.sent()])];
                    case 23:
                        _d.sent();
                        _d.label = 24;
                    case 24: return [2 /*return*/];
                }
            });
        });
    };
    AsyncPatchIterable = tslib_1.__decorate([
        internal_1.ToStringTag("AsyncPatchIterable")
    ], AsyncPatchIterable);
    return AsyncPatchIterable;
}());
