"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var internal_1 = require("../internal");
var toArray_1 = require("./toArray");
var common_1 = require("./common");
function reduceRight(source, accumulator, seed, resultSelector) {
    if (resultSelector === void 0) { resultSelector = common_1.identity; }
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeFunction(accumulator, "accumulator");
    internal_1.assert.mustBeFunction(resultSelector, "resultSelector");
    var sourceArray = toArray_1.toArray(source);
    var hasCurrent = arguments.length > 2;
    var current = seed;
    var count = 0;
    for (var offset = sourceArray.length - 1; offset >= 0; offset--) {
        var value = sourceArray[offset];
        if (!hasCurrent) {
            current = value;
            hasCurrent = true;
        }
        else {
            current = accumulator(current, value, offset);
        }
        count++;
    }
    return resultSelector(current, count);
}
exports.reduceRight = reduceRight;
