"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
/**
 * Creates an Iterable for a value repeated a provided number of times.
 *
 * @param value The value for each element of the Iterable.
 * @param count The number of times to repeat the value.
 * @category Query
 */
function repeat(value, count) {
    internal_1.assert.mustBePositiveFiniteNumber(count, "count");
    return new RepeatIterable(value, count);
}
exports.repeat = repeat;
var RepeatIterable = /** @class */ (function () {
    function RepeatIterable(value, count) {
        this._value = value;
        this._count = count;
    }
    RepeatIterable.prototype[Symbol.iterator] = function () {
        var value, count;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    value = this._value;
                    count = this._count;
                    _a.label = 1;
                case 1:
                    if (!(count > 0)) return [3 /*break*/, 4];
                    return [4 /*yield*/, value];
                case 2:
                    _a.sent();
                    _a.label = 3;
                case 3:
                    --count;
                    return [3 /*break*/, 1];
                case 4: return [2 /*return*/];
            }
        });
    };
    RepeatIterable = tslib_1.__decorate([
        internal_1.ToStringTag("RepeatIterable")
    ], RepeatIterable);
    return RepeatIterable;
}());
