"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var toArray_1 = require("./toArray");
function scanRight(source, accumulator, seed) {
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeFunction(accumulator, "accumulator");
    return new ScanRightIterable(internal_1.ToIterable(source), accumulator, arguments.length > 2, seed);
}
exports.scanRight = scanRight;
var ScanRightIterable = /** @class */ (function () {
    function ScanRightIterable(source, accumulator, isSeeded, seed) {
        this._source = source;
        this._accumulator = accumulator;
        this._isSeeded = isSeeded;
        this._seed = seed;
    }
    ScanRightIterable.prototype[Symbol.iterator] = function () {
        var source, accumulator, hasCurrent, current, offset, value;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    source = toArray_1.toArray(this._source);
                    accumulator = this._accumulator;
                    hasCurrent = this._isSeeded;
                    current = this._seed;
                    offset = source.length - 1;
                    _a.label = 1;
                case 1:
                    if (!(offset >= 0)) return [3 /*break*/, 4];
                    value = source[offset];
                    if (!hasCurrent) {
                        current = value;
                        hasCurrent = true;
                        return [3 /*break*/, 3];
                    }
                    current = accumulator(current, value, offset);
                    return [4 /*yield*/, current];
                case 2:
                    _a.sent();
                    _a.label = 3;
                case 3:
                    offset--;
                    return [3 /*break*/, 1];
                case 4: return [2 /*return*/];
            }
        });
    };
    ScanRightIterable = tslib_1.__decorate([
        internal_1.ToStringTag("ScanRightIterable")
    ], ScanRightIterable);
    return ScanRightIterable;
}());
