"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var toArrayAsync_1 = require("./toArrayAsync");
function scanRightAsync(source, accumulator, seed) {
    internal_1.assert.mustBeAsyncQueryable(source, "source");
    internal_1.assert.mustBeFunction(accumulator, "accumulator");
    return new AsyncScanRightIterable(internal_1.ToPossiblyAsyncIterable(source), accumulator, arguments.length > 2, seed);
}
exports.scanRightAsync = scanRightAsync;
var AsyncScanRightIterable = /** @class */ (function () {
    function AsyncScanRightIterable(source, accumulator, isSeeded, seed) {
        this._source = source;
        this._accumulator = accumulator;
        this._isSeeded = isSeeded;
        this._seed = seed;
    }
    AsyncScanRightIterable.prototype[Symbol.asyncIterator] = function () {
        return tslib_1.__asyncGenerator(this, arguments, function _a() {
            var source, accumulator, hasCurrent, current, offset, value;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, tslib_1.__await(toArrayAsync_1.toArrayAsync(this._source))];
                    case 1:
                        source = _a.sent();
                        accumulator = this._accumulator;
                        hasCurrent = this._isSeeded;
                        current = this._seed;
                        offset = source.length - 1;
                        _a.label = 2;
                    case 2:
                        if (!(offset >= 0)) return [3 /*break*/, 7];
                        value = source[offset];
                        if (!hasCurrent) {
                            current = value;
                            hasCurrent = true;
                            return [3 /*break*/, 6];
                        }
                        return [4 /*yield*/, tslib_1.__await(accumulator(current, value, offset))];
                    case 3:
                        current = _a.sent();
                        return [4 /*yield*/, tslib_1.__await(current)];
                    case 4: return [4 /*yield*/, _a.sent()];
                    case 5:
                        _a.sent();
                        _a.label = 6;
                    case 6:
                        offset--;
                        return [3 /*break*/, 2];
                    case 7: return [2 /*return*/];
                }
            });
        });
    };
    AsyncScanRightIterable = tslib_1.__decorate([
        internal_1.ToStringTag("AsyncScanRightIterable")
    ], AsyncScanRightIterable);
    return AsyncScanRightIterable;
}());
