"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var prepend_1 = require("./prepend");
var consume_1 = require("./consume");
var empty_1 = require("./empty");
var noCacheAndLeaveOpen = { cacheElements: false, leaveOpen: true };
var cacheAndClose = { cacheElements: true, leaveOpen: false };
function span(source, predicate) {
    var e_1, _a;
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeFunction(predicate, "predicate");
    var prefix = [];
    var iterator = internal_1.GetIterator(internal_1.ToIterable(source));
    var offset = 0;
    try {
        for (var _b = tslib_1.__values(consume_1.consume(iterator, noCacheAndLeaveOpen)), _c = _b.next(); !_c.done; _c = _b.next()) {
            var value = _c.value;
            if (!predicate(value, offset++)) {
                var remaining = prepend_1.prepend(consume_1.consume(iterator, cacheAndClose), value);
                return [
                    internal_1.FlowHierarchy(prefix, source),
                    internal_1.FlowHierarchy(remaining, source)
                ];
            }
            prefix.push(value);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return [
        internal_1.FlowHierarchy(prefix, source),
        internal_1.FlowHierarchy(empty_1.empty(), source)
    ];
}
exports.span = span;
