"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var common_1 = require("./common");
function spanMap(source, keySelector, elementSelector, spanSelector) {
    if (elementSelector === void 0) { elementSelector = common_1.identity; }
    if (spanSelector === void 0) { spanSelector = internal_1.CreateGrouping; }
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    internal_1.assert.mustBeFunction(elementSelector, "elementSelector");
    internal_1.assert.mustBeFunction(spanSelector, "spanSelector");
    return new SpanMapIterable(internal_1.ToIterable(source), keySelector, elementSelector, spanSelector);
}
exports.spanMap = spanMap;
var SpanMapIterable = /** @class */ (function () {
    function SpanMapIterable(source, keySelector, elementSelector, spanSelector) {
        this._source = source;
        this._keySelector = keySelector;
        this._elementSelector = elementSelector;
        this._spanSelector = spanSelector;
    }
    SpanMapIterable.prototype[Symbol.iterator] = function () {
        var e_1, _a, source, keySelector, elementSelector, spanSelector, span, previousKey, source_1, source_1_1, element, key, e_1_1;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    source = internal_1.ToIterable(this._source);
                    keySelector = this._keySelector;
                    elementSelector = this._elementSelector;
                    spanSelector = this._spanSelector;
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 8, 9, 10]);
                    source_1 = tslib_1.__values(source), source_1_1 = source_1.next();
                    _b.label = 2;
                case 2:
                    if (!!source_1_1.done) return [3 /*break*/, 7];
                    element = source_1_1.value;
                    key = keySelector(element);
                    if (!!span) return [3 /*break*/, 3];
                    previousKey = key;
                    span = [];
                    return [3 /*break*/, 5];
                case 3:
                    if (!!internal_1.SameValue(previousKey, key)) return [3 /*break*/, 5];
                    return [4 /*yield*/, spanSelector(previousKey, elementSelector === common_1.identity ? internal_1.FlowHierarchy(span, this._source) : span)];
                case 4:
                    _b.sent();
                    span = [];
                    previousKey = key;
                    _b.label = 5;
                case 5:
                    span.push(elementSelector(element));
                    _b.label = 6;
                case 6:
                    source_1_1 = source_1.next();
                    return [3 /*break*/, 2];
                case 7: return [3 /*break*/, 10];
                case 8:
                    e_1_1 = _b.sent();
                    e_1 = { error: e_1_1 };
                    return [3 /*break*/, 10];
                case 9:
                    try {
                        if (source_1_1 && !source_1_1.done && (_a = source_1.return)) _a.call(source_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                    return [7 /*endfinally*/];
                case 10:
                    if (!span) return [3 /*break*/, 12];
                    return [4 /*yield*/, spanSelector(previousKey, elementSelector === common_1.identity ? internal_1.FlowHierarchy(span, this._source) : span)];
                case 11:
                    _b.sent();
                    _b.label = 12;
                case 12: return [2 /*return*/];
            }
        });
    };
    SpanMapIterable = tslib_1.__decorate([
        internal_1.ToStringTag("SpanMapIterable")
    ], SpanMapIterable);
    return SpanMapIterable;
}());
