"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var common_1 = require("./common");
function spanMapAsync(source, keySelector, elementSelector, spanSelector) {
    if (elementSelector === void 0) { elementSelector = common_1.identity; }
    if (spanSelector === void 0) { spanSelector = internal_1.CreateGrouping; }
    internal_1.assert.mustBeAsyncQueryable(source, "source");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    internal_1.assert.mustBeFunction(elementSelector, "elementSelector");
    internal_1.assert.mustBeFunction(spanSelector, "spanSelector");
    return new AsyncSpanMapIterable(internal_1.ToPossiblyAsyncIterable(source), keySelector, elementSelector, spanSelector);
}
exports.spanMapAsync = spanMapAsync;
var AsyncSpanMapIterable = /** @class */ (function () {
    function AsyncSpanMapIterable(source, keySelector, elementSelector, spanSelector) {
        this._source = source;
        this._keySelector = keySelector;
        this._elementSelector = elementSelector;
        this._spanSelector = spanSelector;
    }
    AsyncSpanMapIterable.prototype[Symbol.asyncIterator] = function () {
        return tslib_1.__asyncGenerator(this, arguments, function _a() {
            var e_1, _a, keySelector, elementSelector, spanSelector, span, previousKey, _b, _c, element, key, _d, _e, e_1_1;
            return tslib_1.__generator(this, function (_f) {
                switch (_f.label) {
                    case 0:
                        keySelector = this._keySelector;
                        elementSelector = this._elementSelector;
                        spanSelector = this._spanSelector;
                        _f.label = 1;
                    case 1:
                        _f.trys.push([1, 11, 12, 17]);
                        _b = tslib_1.__asyncValues(this._source);
                        _f.label = 2;
                    case 2: return [4 /*yield*/, tslib_1.__await(_b.next())];
                    case 3:
                        if (!(_c = _f.sent(), !_c.done)) return [3 /*break*/, 10];
                        element = _c.value;
                        key = keySelector(element);
                        if (!!span) return [3 /*break*/, 4];
                        previousKey = key;
                        span = [];
                        return [3 /*break*/, 7];
                    case 4:
                        if (!!internal_1.SameValue(previousKey, key)) return [3 /*break*/, 7];
                        return [4 /*yield*/, tslib_1.__await(spanSelector(previousKey, elementSelector === common_1.identity ? internal_1.FlowHierarchy(span, this._source) : span))];
                    case 5: return [4 /*yield*/, _f.sent()];
                    case 6:
                        _f.sent();
                        span = [];
                        previousKey = key;
                        _f.label = 7;
                    case 7:
                        _e = (_d = span).push;
                        return [4 /*yield*/, tslib_1.__await(elementSelector(element))];
                    case 8:
                        _e.apply(_d, [_f.sent()]);
                        _f.label = 9;
                    case 9: return [3 /*break*/, 2];
                    case 10: return [3 /*break*/, 17];
                    case 11:
                        e_1_1 = _f.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 17];
                    case 12:
                        _f.trys.push([12, , 15, 16]);
                        if (!(_c && !_c.done && (_a = _b.return))) return [3 /*break*/, 14];
                        return [4 /*yield*/, tslib_1.__await(_a.call(_b))];
                    case 13:
                        _f.sent();
                        _f.label = 14;
                    case 14: return [3 /*break*/, 16];
                    case 15:
                        if (e_1) throw e_1.error;
                        return [7 /*endfinally*/];
                    case 16: return [7 /*endfinally*/];
                    case 17:
                        if (!span) return [3 /*break*/, 20];
                        return [4 /*yield*/, tslib_1.__await(spanSelector(previousKey, elementSelector === common_1.identity ? internal_1.FlowHierarchy(span, this._source) : span))];
                    case 18: return [4 /*yield*/, _f.sent()];
                    case 19:
                        _f.sent();
                        _f.label = 20;
                    case 20: return [2 /*return*/];
                }
            });
        });
    };
    AsyncSpanMapIterable = tslib_1.__decorate([
        internal_1.ToStringTag("AsyncSpanMapIterable")
    ], AsyncSpanMapIterable);
    return AsyncSpanMapIterable;
}());
