"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var internal_1 = require("../internal");
function startsWith(left, right, equalityComparison) {
    if (equalityComparison === void 0) { equalityComparison = internal_1.SameValue; }
    var _a, _b;
    internal_1.assert.mustBeQueryable(left, "left");
    internal_1.assert.mustBeQueryable(right, "right");
    internal_1.assert.mustBeFunction(equalityComparison, "equalityComparison");
    var leftIterator = internal_1.GetIterator(internal_1.ToIterable(left));
    var leftDone = false;
    var leftValue;
    try {
        var rightIterator = internal_1.GetIterator(internal_1.ToIterable(right));
        var rightDone = false;
        var rightValue = void 0;
        try {
            for (;;) {
                (_a = leftIterator.next(), leftDone = _a.done, leftValue = _a.value);
                (_b = rightIterator.next(), rightDone = _b.done, rightValue = _b.value);
                if (rightDone)
                    return true;
                if (leftDone || !equalityComparison(leftValue, rightValue))
                    return false;
            }
        }
        finally {
            if (!rightDone)
                internal_1.IteratorClose(rightIterator);
        }
    }
    finally {
        if (!leftDone)
            internal_1.IteratorClose(leftIterator);
    }
}
exports.startsWith = startsWith;
