/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
import { Queryable, HierarchyIterable } from "../types";
/**
 * Creates a subquery for the symmetric difference between two [[Queryable]] objects, where set identity is determined by the selected key.
 * The result is an [[Iterable]] containings the elements that exist in only left or right, but not
 * in both.
 *
 * @param left A [[Queryable]] object.
 * @param right A [[Queryable]] object.
 * @param keySelector A callback used to select the key for each element.
 * @category Subquery
 */
export declare function symmetricDifferenceBy<TNode, T extends TNode, K>(left: HierarchyIterable<TNode, T>, right: Queryable<T>, keySelector: (element: T) => K): HierarchyIterable<TNode, T>;
/**
 * Creates a subquery for the symmetric difference between two [[Queryable]] objects, where set identity is determined by the selected key.
 * The result is an [[Iterable]] containings the elements that exist in only left or right, but not
 * in both.
 *
 * @param left A [[Queryable]] object.
 * @param right A [[Queryable]] object.
 * @param keySelector A callback used to select the key for each element.
 * @category Subquery
 */
export declare function symmetricDifferenceBy<TNode, T extends TNode, K>(left: Queryable<T>, right: HierarchyIterable<TNode, T>, keySelector: (element: T) => K): HierarchyIterable<TNode, T>;
/**
 * Creates a subquery for the symmetric difference between two [[Queryable]] objects, where set identity is determined by the selected key.
 * The result is an [[Iterable]] containings the elements that exist in only left or right, but not
 * in both.
 *
 * @param left A [[Queryable]] object.
 * @param right A [[Queryable]] object.
 * @param keySelector A callback used to select the key for each element.
 * @category Subquery
 */
export declare function symmetricDifferenceBy<T, K>(left: Queryable<T>, right: Queryable<T>, keySelector: (element: T) => K): Iterable<T>;
