"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var common_1 = require("./common");
function zip(left, right, selector) {
    if (selector === void 0) { selector = common_1.tuple; }
    internal_1.assert.mustBeQueryable(left, "left");
    internal_1.assert.mustBeQueryable(right, "right");
    internal_1.assert.mustBeFunction(selector, "selector");
    return new ZipIterable(internal_1.ToIterable(left), internal_1.ToIterable(right), selector);
}
exports.zip = zip;
var ZipIterable = /** @class */ (function () {
    function ZipIterable(left, right, selector) {
        this._left = left;
        this._right = right;
        this._selector = selector;
    }
    ZipIterable.prototype[Symbol.iterator] = function () {
        var _a, _b, selector, leftIterator, leftDone, leftValue, rightIterator, rightDone, rightValue;
        return tslib_1.__generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    selector = this._selector;
                    leftIterator = internal_1.GetIterator(this._left);
                    leftDone = false;
                    _c.label = 1;
                case 1:
                    _c.trys.push([1, , 9, 10]);
                    rightIterator = internal_1.GetIterator(this._right);
                    rightDone = false;
                    rightValue = void 0;
                    _c.label = 2;
                case 2:
                    _c.trys.push([2, , 7, 8]);
                    _c.label = 3;
                case 3:
                    (_a = leftIterator.next(), leftDone = _a.done, leftValue = _a.value);
                    (_b = rightIterator.next(), rightDone = _b.done, rightValue = _b.value);
                    if (leftDone || rightDone)
                        return [3 /*break*/, 6];
                    return [4 /*yield*/, selector(leftValue, rightValue)];
                case 4:
                    _c.sent();
                    _c.label = 5;
                case 5: return [3 /*break*/, 3];
                case 6: return [3 /*break*/, 8];
                case 7:
                    if (!rightDone)
                        internal_1.IteratorClose(rightIterator);
                    return [7 /*endfinally*/];
                case 8: return [3 /*break*/, 10];
                case 9:
                    if (!leftDone)
                        internal_1.IteratorClose(leftIterator);
                    return [7 /*endfinally*/];
                case 10: return [2 /*return*/];
            }
        });
    };
    ZipIterable = tslib_1.__decorate([
        internal_1.ToStringTag("ZipIterable")
    ], ZipIterable);
    return ZipIterable;
}());
