"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Lazy = /** @class */ (function () {
    function Lazy() {
        this._value = undefined;
        this._factory = undefined;
        this._args = undefined;
    }
    Lazy.from = function (factory) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var lazy = new Lazy();
        lazy._factory = factory;
        lazy._args = args;
        return lazy;
    };
    Lazy.for = function (value) {
        var lazy = new Lazy();
        lazy._value = value;
        return lazy;
    };
    Object.defineProperty(Lazy.prototype, "hasValue", {
        get: function () {
            return this._factory === undefined;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Lazy.prototype, "value", {
        get: function () {
            var factory = this._factory;
            var args = this._args;
            if (factory) {
                var ok = false;
                try {
                    this._factory = resolvingValueFactory;
                    this._args = undefined;
                    this._value = args ? factory.apply(void 0, tslib_1.__spread(args)) : factory();
                    ok = true;
                }
                finally {
                    if (ok) {
                        this._factory = undefined;
                    }
                    else {
                        this._factory = factory;
                        this._args = args;
                    }
                }
            }
            return this._value;
        },
        enumerable: true,
        configurable: true
    });
    return Lazy;
}());
exports.Lazy = Lazy;
function resolvingValueFactory() {
    throw new Error("'value' recursively references itself during its own initialization.");
}
