"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
/**
 * Writes each element of a source iterable to a destination array.
 *
 * @param source A [[Queryable]] object.
 * @param dest The destination array.
 * @param start The offset into the array at which to start writing.
 * @param count The number of elements to write to the array.
 * @category Scalar
 */
async function copyToAsync(source, dest, start = 0, count = dest.length - start) {
    var e_1, _a;
    internal_1.assert.mustBeAsyncQueryable(source, "source");
    internal_1.assert.mustBeArrayLike(dest, "dest");
    internal_1.assert.mustBePositiveInteger(start, "start");
    internal_1.assert.mustBePositiveInteger(count, "count");
    if (count > 0) {
        const minLength = start + count;
        try {
            for (var _b = tslib_1.__asyncValues(internal_1.ToPossiblyAsyncIterable(source)), _c; _c = await _b.next(), !_c.done;) {
                const element = _c.value;
                if (count > 0) {
                    dest[start++] = element;
                    count--;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) await _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        if (dest.length < minLength)
            dest.length = minLength;
    }
    return dest;
}
exports.copyToAsync = copyToAsync;
