"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const internal_1 = require("../internal");
async function correspondsByAsync(left, right, leftKeySelector, rightKeySelector = leftKeySelector, equalityComparison = internal_1.SameValueZero) {
    internal_1.assert.mustBeAsyncQueryable(left, "left");
    internal_1.assert.mustBeAsyncQueryable(right, "right");
    internal_1.assert.mustBeFunction(equalityComparison, "equalityComparison");
    const leftIterator = internal_1.GetAsyncIterator(internal_1.ToPossiblyAsyncIterable(left));
    let leftDone = false;
    let leftValue;
    try {
        const rightIterator = internal_1.GetAsyncIterator(internal_1.ToPossiblyAsyncIterable(right));
        let rightDone = false;
        let rightValue;
        try {
            for (;;) {
                ({ done: leftDone, value: leftValue } = await leftIterator.next());
                ({ done: rightDone, value: rightValue } = await rightIterator.next());
                if (leftDone && rightDone)
                    return true;
                if (Boolean(leftDone) !== Boolean(rightDone) || !equalityComparison(leftKeySelector(leftValue), rightKeySelector(rightValue)))
                    return false;
            }
        }
        finally {
            if (!rightDone)
                await internal_1.AsyncIteratorClose(rightIterator);
        }
    }
    finally {
        if (!leftDone)
            await internal_1.AsyncIteratorClose(leftIterator);
    }
}
exports.correspondsByAsync = correspondsByAsync;
