"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const internal_1 = require("../internal");
const last_1 = require("./last");
/**
 * Finds the value at the provided offset. A negative offset starts from the
 * last element.
 *
 * @param source A [[Queryable]] object.
 * @param offset An offset.
 * @category Scalar
 */
function elementAt(source, offset) {
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeInteger(offset, "offset");
    if (offset === -1) {
        return last_1.last(source);
    }
    if (offset < 0) {
        offset = Math.abs(offset);
        const array = [];
        for (const element of internal_1.ToIterable(source)) {
            if (array.length >= offset) {
                array.shift();
            }
            array.push(element);
        }
        return array.length - offset >= 0 ? array[array.length - offset] : undefined;
    }
    for (const element of internal_1.ToIterable(source)) {
        if (offset === 0) {
            return element;
        }
        offset--;
    }
    return undefined;
}
exports.elementAt = elementAt;
