"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
function expandAsync(source, projection) {
    internal_1.assert.mustBeAsyncQueryable(source, "source");
    internal_1.assert.mustBeFunction(projection, "projection");
    return internal_1.FlowHierarchy(new AsyncExpandIterable(internal_1.ToPossiblyAsyncIterable(source), projection), source);
}
exports.expandAsync = expandAsync;
let AsyncExpandIterable = class AsyncExpandIterable {
    constructor(source, projection) {
        this._source = source;
        this._projection = projection;
    }
    [Symbol.asyncIterator]() {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            var e_1, _a;
            const projection = this._projection;
            const queue = [this._source];
            while (queue.length) {
                const source = queue.shift();
                try {
                    for (var source_1 = tslib_1.__asyncValues(source), source_1_1; source_1_1 = yield tslib_1.__await(source_1.next()), !source_1_1.done;) {
                        const element = source_1_1.value;
                        const projected = projection(element);
                        internal_1.assert.mustBeAsyncQueryable(projected);
                        queue.push(internal_1.ToPossiblyAsyncIterable(projected));
                        yield yield tslib_1.__await(element);
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (source_1_1 && !source_1_1.done && (_a = source_1.return)) yield tslib_1.__await(_a.call(source_1));
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            }
        });
    }
};
AsyncExpandIterable = tslib_1.__decorate([
    internal_1.ToStringTag("AsyncExpandIterable")
], AsyncExpandIterable);
