"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const internal_1 = require("../internal");
const common_1 = require("./common");
/**
 * Gets the first element, optionally filtering elements using the supplied callback.
 *
 * @param source A [[Queryable]] object.
 * @param predicate An optional callback used to match each element.
 * @category Scalar
 */
function first(source, predicate = common_1.T) {
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeFunction(predicate, "predicate");
    for (const element of internal_1.ToIterable(source)) {
        if (predicate(element)) {
            return element;
        }
    }
    return undefined;
}
exports.first = first;
