"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
function flatMapAsync(source, projection, resultSelector) {
    internal_1.assert.mustBeAsyncQueryable(source, "source");
    internal_1.assert.mustBeFunction(projection, "projection");
    return new AsyncFlatMapIterable(internal_1.ToPossiblyAsyncIterable(source), projection, resultSelector);
}
exports.flatMapAsync = flatMapAsync;
let AsyncFlatMapIterable = class AsyncFlatMapIterable {
    constructor(source, projection, resultSelector) {
        this._source = source;
        this._projection = projection;
        this._resultSelector = resultSelector;
    }
    [Symbol.asyncIterator]() {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            var e_1, _a, e_2, _b;
            const projection = this._projection;
            const resultSelector = this._resultSelector;
            try {
                for (var _c = tslib_1.__asyncValues(this._source), _d; _d = yield tslib_1.__await(_c.next()), !_d.done;) {
                    const element = _d.value;
                    const inner = projection(element);
                    if (resultSelector) {
                        try {
                            for (var _e = tslib_1.__asyncValues(internal_1.ToPossiblyAsyncIterable(inner)), _f; _f = yield tslib_1.__await(_e.next()), !_f.done;) {
                                const innerElement = _f.value;
                                yield yield tslib_1.__await(resultSelector(element, innerElement));
                            }
                        }
                        catch (e_2_1) { e_2 = { error: e_2_1 }; }
                        finally {
                            try {
                                if (_f && !_f.done && (_b = _e.return)) yield tslib_1.__await(_b.call(_e));
                            }
                            finally { if (e_2) throw e_2.error; }
                        }
                    }
                    else {
                        yield tslib_1.__await(yield* tslib_1.__asyncDelegator(tslib_1.__asyncValues(internal_1.ToPossiblyAsyncIterable(inner))));
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_d && !_d.done && (_a = _c.return)) yield tslib_1.__await(_a.call(_c));
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
};
AsyncFlatMapIterable = tslib_1.__decorate([
    internal_1.ToStringTag("AsyncFlatMapIterable")
], AsyncFlatMapIterable);
