"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const internal_1 = require("../internal");
const toArray_1 = require("./toArray");
function includesSequence(left, right, equalityComparison = internal_1.SameValue) {
    internal_1.assert.mustBeQueryable(left, "source");
    internal_1.assert.mustBeQueryable(right, "other");
    internal_1.assert.mustBeFunction(equalityComparison, "equalityComparison");
    const rightArray = toArray_1.toArray(right);
    const numRightElements = rightArray.length;
    if (numRightElements <= 0) {
        return true;
    }
    const span = [];
    for (const leftValue of internal_1.ToIterable(left)) {
        for (;;) {
            const rightValue = rightArray[span.length];
            if (equalityComparison(leftValue, rightValue)) {
                if (span.length + 1 >= numRightElements) {
                    return true;
                }
                span.push(leftValue);
            }
            else if (span.length > 0) {
                span.shift();
                continue;
            }
            break;
        }
    }
    return false;
}
exports.includesSequence = includesSequence;
