/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
import { Queryable, HierarchyIterable } from "../types";
/**
 * Creates a [[HierarchyIterable]] for the set intersection of a [[HierarchyIterable]] object and a [[Queryable]] object.
 *
 * @param left A [[HierarchyIterable]] object.
 * @param right A [[Queryable]] object.
 * @category Subquery
 */
export declare function intersect<TNode, T extends TNode>(left: HierarchyIterable<TNode, T>, right: Queryable<T>): HierarchyIterable<TNode, T>;
/**
 * Creates a [[HierarchyIterable]] for the set intersection of a [[Queryable]] object and a [[HierarchyIterable]] object.
 *
 * @param left A [[Queryable]] object.
 * @param right A [[HierarchyIterable]] object.
 * @category Subquery
 */
export declare function intersect<TNode, T extends TNode>(left: Queryable<T>, right: HierarchyIterable<TNode, T>): HierarchyIterable<TNode, T>;
/**
 * Creates an [[Iterable]] for the set intersection of two [[Queryable]] objects.
 *
 * @param left A [[Queryable]] object.
 * @param right A [[Queryable]] object.
 * @category Subquery
 */
export declare function intersect<T>(left: Queryable<T>, right: Queryable<T>): Iterable<T>;
