"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const types_1 = require("../types");
const axis_1 = require("./axis");
const elementAt_1 = require("./elementAt");
/**
 * Creates an [[AsyncHierarchyIterable]] for the child of each element at the specified offset. A negative offset
 * starts from the last child.
 *
 * @param source A [[HierarchyIterable]] or [[AsyncHierarchyIterable]] object.
 * @param offset The offset for the child.
 * @category Hierarchy
 */
function nthChildAsync(source, offset) {
    internal_1.assert.mustBePossiblyAsyncHierarchyIterable(source, "source");
    internal_1.assert.mustBeInteger(offset, "offset");
    return new AsyncNthChildIterable(source, offset);
}
exports.nthChildAsync = nthChildAsync;
function nthChildOf(element, provider, offset) {
    if (element !== undefined) {
        const children = axis_1.Axis.children(provider, element);
        return elementAt_1.elementAt(children, offset);
    }
    return undefined;
}
let AsyncNthChildIterable = class AsyncNthChildIterable {
    constructor(source, offset) {
        this._source = source;
        this._offset = offset;
    }
    [Symbol.asyncIterator]() {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            var e_1, _a;
            const offset = this._offset;
            const hierarchy = internal_1.GetHierarchy(this._source);
            try {
                for (var _b = tslib_1.__asyncValues(this._source), _c; _c = yield tslib_1.__await(_b.next()), !_c.done;) {
                    const element = _c.value;
                    const child = nthChildOf(element, hierarchy, offset);
                    if (child !== undefined) {
                        yield yield tslib_1.__await(child);
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) yield tslib_1.__await(_a.call(_b));
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
    [types_1.Hierarchical.hierarchy]() {
        return internal_1.GetHierarchy(this._source);
    }
};
AsyncNthChildIterable = tslib_1.__decorate([
    internal_1.ToStringTag("AsyncNthChildIterable")
], AsyncNthChildIterable);
