"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
function pageBy(source, pageSize) {
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBePositiveNonZeroFiniteNumber(pageSize, "pageSize");
    return new PageByIterable(internal_1.ToIterable(source), pageSize);
}
exports.pageBy = pageBy;
let PageByIterable = class PageByIterable {
    constructor(source, pageSize) {
        this._source = source;
        this._pageSize = pageSize;
    }
    *[Symbol.iterator]() {
        const pageSize = this._pageSize;
        let elements = [];
        let page = 0;
        for (const value of this._source) {
            elements.push(value);
            if (elements.length >= pageSize) {
                yield internal_1.CreatePage(page, page * pageSize, elements);
                elements = [];
                page++;
            }
        }
        if (elements.length > 0) {
            yield internal_1.CreatePage(page, page * pageSize, elements);
        }
    }
};
PageByIterable = tslib_1.__decorate([
    internal_1.ToStringTag("PageByIterable")
], PageByIterable);
