/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
import { AsyncQueryable, PossiblyAsyncHierarchyIterable, AsyncHierarchyIterable } from "../types";
/**
 * Creates an [[AsyncHierarchyIterable]] for the elements of the source with the provided range
 * patched into the results.
 *
 * @param source A [[HierarchyIterable]] or [[AsyncHierarchyIterable]] to patch.
 * @param start The offset at which to patch the range.
 * @param skipCount The number of elements to skip from start.
 * @param range The range to patch into the result.
 * @category Subquery
 */
export declare function patchAsync<TNode, T extends TNode>(source: PossiblyAsyncHierarchyIterable<TNode, T>, start: number, skipCount?: number, range?: AsyncQueryable<T>): AsyncHierarchyIterable<TNode, T>;
/**
 * Creates an [[AsyncIterable]] for the elements of the source with the provided range
 * patched into the results.
 *
 * @param source An [[AsyncQueryable]] to patch.
 * @param start The offset at which to patch the range.
 * @param skipCount The number of elements to skip from start.
 * @param range The range to patch into the result.
 * @category Subquery
 */
export declare function patchAsync<T>(source: AsyncQueryable<T>, start: number, skipCount?: number, range?: AsyncQueryable<T>): AsyncIterable<T>;
