"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
/**
 * Creates an Iterable for a value repeated a provided number of times.
 *
 * @param value The value for each element of the Iterable.
 * @param count The number of times to repeat the value.
 * @category Query
 */
function repeatAsync(value, count) {
    internal_1.assert.mustBePositiveFiniteNumber(count, "count");
    return new AsyncRepeatIterable(value, count);
}
exports.repeatAsync = repeatAsync;
let AsyncRepeatIterable = class AsyncRepeatIterable {
    constructor(value, count) {
        this._value = value;
        this._count = count;
    }
    [Symbol.asyncIterator]() {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            const value = yield tslib_1.__await(this._value);
            let count = this._count;
            while (count > 0) {
                yield yield tslib_1.__await(value);
                count--;
            }
        });
    }
};
AsyncRepeatIterable = tslib_1.__decorate([
    internal_1.ToStringTag("AsyncRepeatIterable")
], AsyncRepeatIterable);
