"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
function scan(source, accumulator, seed) {
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeFunction(accumulator, "accumulator");
    return new ScanIterable(internal_1.ToIterable(source), accumulator, arguments.length > 2, seed);
}
exports.scan = scan;
let ScanIterable = class ScanIterable {
    constructor(source, accumulator, isSeeded, seed) {
        this._source = source;
        this._accumulator = accumulator;
        this._isSeeded = isSeeded;
        this._seed = seed;
    }
    *[Symbol.iterator]() {
        const accumulator = this._accumulator;
        let hasCurrent = this._isSeeded;
        let current = this._seed;
        let offset = 0;
        for (const value of this._source) {
            if (!hasCurrent) {
                current = value;
                hasCurrent = true;
            }
            else {
                current = accumulator(current, value, offset);
                yield current;
            }
            offset++;
        }
    }
};
ScanIterable = tslib_1.__decorate([
    internal_1.ToStringTag("ScanIterable")
], ScanIterable);
