/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
import { AsyncQueryable } from "../types";
/**
 * Creates a subquery containing the cumulative results of applying the provided callback to each element.
 *
 * @param source An [[AsyncQueryable]] object.
 * @param accumulator The callback used to compute each result.
 * @category Subquery
 */
export declare function scanAsync<T>(source: AsyncQueryable<T>, accumulator: (current: T, element: T, offset: number) => T | PromiseLike<T>): AsyncIterable<T>;
/**
 * Creates a subquery containing the cumulative results of applying the provided callback to each element.
 *
 * @param source An [[AsyncQueryable]] object.
 * @param accumulator The callback used to compute each result.
 * @param seed An optional seed value.
 * @category Subquery
 */
export declare function scanAsync<T, U>(source: AsyncQueryable<T>, accumulator: (current: U, element: T, offset: number) => U | PromiseLike<U>, seed: U): AsyncIterable<U>;
