"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
function scanAsync(source, accumulator, seed) {
    internal_1.assert.mustBeAsyncQueryable(source, "source");
    internal_1.assert.mustBeFunction(accumulator, "accumulator");
    return new AsyncScanIterable(internal_1.ToPossiblyAsyncIterable(source), accumulator, arguments.length > 2, seed);
}
exports.scanAsync = scanAsync;
let AsyncScanIterable = class AsyncScanIterable {
    constructor(source, accumulator, isSeeded, seed) {
        this._source = source;
        this._accumulator = accumulator;
        this._isSeeded = isSeeded;
        this._seed = seed;
    }
    [Symbol.asyncIterator]() {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            var e_1, _a;
            const accumulator = this._accumulator;
            let hasCurrent = this._isSeeded;
            let current = this._seed;
            let offset = 0;
            try {
                for (var _b = tslib_1.__asyncValues(this._source), _c; _c = yield tslib_1.__await(_b.next()), !_c.done;) {
                    const value = _c.value;
                    if (!hasCurrent) {
                        current = value;
                        hasCurrent = true;
                    }
                    else {
                        current = yield tslib_1.__await(accumulator(current, value, offset));
                        yield yield tslib_1.__await(current);
                    }
                    offset++;
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) yield tslib_1.__await(_a.call(_b));
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
};
AsyncScanIterable = tslib_1.__decorate([
    internal_1.ToStringTag("AsyncScanIterable")
], AsyncScanIterable);
