"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const prependAsync_1 = require("./prependAsync");
const consumeAsync_1 = require("./consumeAsync");
const emptyAsync_1 = require("./emptyAsync");
const noCacheAndLeaveOpen = { cacheElements: false, leaveOpen: true };
const cacheAndClose = { cacheElements: true, leaveOpen: false };
async function spanAsync(source, predicate) {
    var e_1, _a;
    internal_1.assert.mustBeAsyncQueryable(source, "source");
    internal_1.assert.mustBeFunction(predicate, "predicate");
    const prefix = [];
    const iterator = internal_1.GetAsyncIterator(internal_1.ToAsyncIterable(source));
    let offset = 0;
    try {
        for (var _b = tslib_1.__asyncValues(consumeAsync_1.consumeAsync(iterator, noCacheAndLeaveOpen)), _c; _c = await _b.next(), !_c.done;) {
            const value = _c.value;
            const result = predicate(value, offset++);
            if (!(typeof result === "boolean" ? result : await result)) {
                const remaining = prependAsync_1.prependAsync(consumeAsync_1.consumeAsync(iterator, cacheAndClose), value);
                return [
                    internal_1.FlowHierarchy(prefix, source),
                    internal_1.FlowHierarchy(remaining, source),
                ];
            }
            prefix.push(value);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) await _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return [
        internal_1.FlowHierarchy(prefix, source),
        internal_1.FlowHierarchy(emptyAsync_1.emptyAsync(), source),
    ];
}
exports.spanAsync = spanAsync;
