"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const internal_1 = require("../internal");
function startsWith(left, right, equalityComparison = internal_1.SameValue) {
    internal_1.assert.mustBeQueryable(left, "left");
    internal_1.assert.mustBeQueryable(right, "right");
    internal_1.assert.mustBeFunction(equalityComparison, "equalityComparison");
    const leftIterator = internal_1.GetIterator(internal_1.ToIterable(left));
    let leftDone = false;
    let leftValue;
    try {
        const rightIterator = internal_1.GetIterator(internal_1.ToIterable(right));
        let rightDone = false;
        let rightValue;
        try {
            for (;;) {
                ({ done: leftDone, value: leftValue } = leftIterator.next());
                ({ done: rightDone, value: rightValue } = rightIterator.next());
                if (rightDone)
                    return true;
                if (leftDone || !equalityComparison(leftValue, rightValue))
                    return false;
            }
        }
        finally {
            if (!rightDone)
                internal_1.IteratorClose(rightIterator);
        }
    }
    finally {
        if (!leftDone)
            internal_1.IteratorClose(leftIterator);
    }
}
exports.startsWith = startsWith;
