"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
function takeRightAsync(source, count) {
    internal_1.assert.mustBeAsyncQueryable(source, "source");
    internal_1.assert.mustBePositiveFiniteNumber(count, "count");
    return internal_1.FlowHierarchy(new AsyncTakeRightIterable(internal_1.ToPossiblyAsyncIterable(source), count), source);
}
exports.takeRightAsync = takeRightAsync;
let AsyncTakeRightIterable = class AsyncTakeRightIterable {
    constructor(source, count) {
        this._source = source;
        this._count = count;
    }
    [Symbol.asyncIterator]() {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            var e_1, _a;
            const count = this._count;
            if (count <= 0) {
                return yield tslib_1.__await(void 0);
            }
            else {
                const pending = [];
                try {
                    for (var _b = tslib_1.__asyncValues(this._source), _c; _c = yield tslib_1.__await(_b.next()), !_c.done;) {
                        const element = _c.value;
                        pending.push(element);
                        if (pending.length > count) {
                            pending.shift();
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return)) yield tslib_1.__await(_a.call(_b));
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                yield tslib_1.__await(yield* tslib_1.__asyncDelegator(tslib_1.__asyncValues(pending)));
            }
        });
    }
};
AsyncTakeRightIterable = tslib_1.__decorate([
    internal_1.ToStringTag("AsyncTakeRightIterable")
], AsyncTakeRightIterable);
