"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
function tap(source, callback) {
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeFunction(callback, "callback");
    return internal_1.FlowHierarchy(new TapIterable(internal_1.ToIterable(source), callback), source);
}
exports.tap = tap;
let TapIterable = class TapIterable {
    constructor(source, callback) {
        this._source = source;
        this._callback = callback;
    }
    *[Symbol.iterator]() {
        const source = this._source;
        const callback = this._callback;
        let offset = 0;
        for (const element of source) {
            callback(element, offset++);
            yield element;
        }
    }
};
TapIterable = tslib_1.__decorate([
    internal_1.ToStringTag("TapIterable")
], TapIterable);
