"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const common_1 = require("./common");
async function toObjectAsync(source, prototype, keySelector, elementSelector = common_1.identity) {
    var e_1, _a;
    internal_1.assert.mustBeAsyncQueryable(source, "source");
    internal_1.assert.mustBeObjectOrNull(prototype, "prototype");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    internal_1.assert.mustBeFunction(elementSelector, "elementSelector");
    const obj = Object.create(prototype);
    try {
        for (var _b = tslib_1.__asyncValues(internal_1.ToPossiblyAsyncIterable(source)), _c; _c = await _b.next(), !_c.done;) {
            const item = _c.value;
            const key = keySelector(item);
            const element = await elementSelector(item);
            obj[key] = element;
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) await _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return obj;
}
exports.toObjectAsync = toObjectAsync;
