"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const collections_1 = require("../collections");
function unionBy(left, right, keySelector) {
    internal_1.assert.mustBeQueryable(left, "left");
    internal_1.assert.mustBeQueryable(right, "right");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    return internal_1.FlowHierarchy(new UnionByIterable(internal_1.ToIterable(left), internal_1.ToIterable(right), keySelector), left, right);
}
exports.unionBy = unionBy;
let UnionByIterable = class UnionByIterable {
    constructor(left, right, keySelector) {
        this._left = left;
        this._right = right;
        this._keySelector = keySelector;
    }
    *[Symbol.iterator]() {
        const keySelector = this._keySelector;
        const set = new collections_1.Set();
        for (const element of this._left) {
            if (internal_1.TryAdd(set, keySelector(element))) {
                yield element;
            }
        }
        for (const element of this._right) {
            if (internal_1.TryAdd(set, keySelector(element))) {
                yield element;
            }
        }
    }
};
UnionByIterable = tslib_1.__decorate([
    internal_1.ToStringTag("UnionByIterable")
], UnionByIterable);
