"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const collections_1 = require("../collections");
function unionByAsync(left, right, keySelector) {
    internal_1.assert.mustBeAsyncQueryable(left, "left");
    internal_1.assert.mustBeAsyncQueryable(right, "right");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    return internal_1.FlowHierarchy(new AsyncUnionByIterable(internal_1.ToPossiblyAsyncIterable(left), internal_1.ToPossiblyAsyncIterable(right), keySelector), left, right);
}
exports.unionByAsync = unionByAsync;
let AsyncUnionByIterable = class AsyncUnionByIterable {
    constructor(left, right, keySelector) {
        this._left = left;
        this._right = right;
        this._keySelector = keySelector;
    }
    [Symbol.asyncIterator]() {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            var e_1, _a, e_2, _b;
            const keySelector = this._keySelector;
            const set = new collections_1.Set();
            try {
                for (var _c = tslib_1.__asyncValues(this._left), _d; _d = yield tslib_1.__await(_c.next()), !_d.done;) {
                    const element = _d.value;
                    if (internal_1.TryAdd(set, keySelector(element))) {
                        yield yield tslib_1.__await(element);
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_d && !_d.done && (_a = _c.return)) yield tslib_1.__await(_a.call(_c));
                }
                finally { if (e_1) throw e_1.error; }
            }
            try {
                for (var _e = tslib_1.__asyncValues(this._right), _f; _f = yield tslib_1.__await(_e.next()), !_f.done;) {
                    const element = _f.value;
                    if (internal_1.TryAdd(set, keySelector(element))) {
                        yield yield tslib_1.__await(element);
                    }
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (_f && !_f.done && (_b = _e.return)) yield tslib_1.__await(_b.call(_e));
                }
                finally { if (e_2) throw e_2.error; }
            }
        });
    }
};
AsyncUnionByIterable = tslib_1.__decorate([
    internal_1.ToStringTag("AsyncUnionByIterable")
], AsyncUnionByIterable);
