"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query" */
Object.defineProperty(exports, "__esModule", { value: true });
class Lazy {
    constructor() {
        this._value = undefined;
        this._factory = undefined;
        this._args = undefined;
    }
    static from(factory, ...args) {
        const lazy = new Lazy();
        lazy._factory = factory;
        lazy._args = args;
        return lazy;
    }
    static for(value) {
        const lazy = new Lazy();
        lazy._value = value;
        return lazy;
    }
    get hasValue() {
        return this._factory === undefined;
    }
    get value() {
        const factory = this._factory;
        const args = this._args;
        if (factory) {
            let ok = false;
            try {
                this._factory = resolvingValueFactory;
                this._args = undefined;
                this._value = args ? factory(...args) : factory();
                ok = true;
            }
            finally {
                if (ok) {
                    this._factory = undefined;
                }
                else {
                    this._factory = factory;
                    this._args = args;
                }
            }
        }
        return this._value;
    }
}
exports.Lazy = Lazy;
function resolvingValueFactory() {
    throw new Error("'value' recursively references itself during its own initialization.");
}
